;***************************************************************************
;
; File Name		:'Main.asm"
; Title			:DataLog Boot Program & MiniTerminal
; Date			:2003.04.28.
; Version		:1.0.1
; Support telephone	:+36-70-333-4034 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:ATmega128
;
;***************************************************************************
;	D E S C R I P T I O N
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  	why
;	----	----------  ---		--------------------------------------------
;	0.01	2002.06.21  VFX		Creation
;	1.00	2003.02.08  VFX		Redesign to PDG041
;	1.01	2003.04.28  VFX		Redesign to DataLog
;
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	Oscillator: f=external, 16.0000 MHz (T=62.5ns)
;*
;***************************************************************************
;
;
;
.include "m128def.inc"
.include "error.inc"

;***************************************************************************
;* Const Def

.equ	TRUE	=0FFh		;Yes
.equ	FALSE	=00h		;No

.equ	MAJOR_REV	='1'
.equ	MINOR_REV	='0'
.equ	MINOR_REVB	='1'

;System

.equ	SYSCLK = 16000000	;System Clock [Hz]
.equ	BaudSpeed1=38400	;speed of UART1
.equ	BaudSpeed0=38400	;speed of UART0

.equ	SYSTACK = 100		;100Hz system interrupt

.EQU	LEDPNormal = 12		;LED villogasi sebessege 4db/s



.equ	PowerTurnOFF = 0xa0

.equ	TouchPress   = 0xA1
.equ	TouchMove    = 0xA2
.equ	TouchRelease = 0xA3



.equ	LCDBackLightTime = 9000		;Light off timer:  1.5 min


;**************************************************************************
;* Hardware Def.
;
.EQU	LED1_DIR = DDRD			;Green LED
.EQU	LED1_PORT= PORTD
.EQU	LED1	  = 6

.EQU	LED0_DIR = DDRD			;Red LED
.EQU	LED0_PORT= PORTD
.EQU	LED0 	  = 4


.equ	ADR_SMDATA = 0x3F00		;SmartMedia Data Register
.equ	ADR_SMMODE = 0x3F01		;SmartMedia Mode Register
.equ	ADR_SMSTAT = 0x3F01		;SmartMedia Mode Register
.equ	ADR_SERA   = 0x3F02		;SerialNumber0
.equ	ADR_SERB   = 0x3F03		;SerialNumber1
.equ	ADR_SERC   = 0x3F04		;SerialNumber2
.equ	ADR_SERD   = 0x3F05		;SerialNumber3
.equ	ADR_ECC0   = 0x3F06		;ECC0
.equ	ADR_ECC1   = 0x3F07		;ECC1
.equ	ADR_ECC2   = 0x3F08		;ECC2
.equ	ADR_LPTD   = 0x3F09		;LPT Data byte
.equ	ADR_LPTS   = 0x3F0A		;LPT Status byte
.equ	ADR_LPTC   = 0x3F0B		;LPT Command byte
.equ	ADR_EXLH   = 0x3F0C		;External Port LH reg
.equ	ADR_EXHL   = 0x3F0D		;External Port HL reg
.equ	ADR_EXHH   = 0x3F0E		;External Port HH reg
.equ	ADR_GSM	   = 0x3F0F		;GSM Modem Port
.equ	ADR_DMAL   = 0x3F10		;DMA Lenght Low
.equ	ADR_DMAH   = 0x3F11		;DMA Lenght High & Start DMA Cycles
.equ	ADR_GPSD   = 0x3F12		;GPS UART Data
.equ	ADR_GPSS   = 0x3F13		;GPS UART Status
.equ	ADR_SPID   = 0x3F14		;AVR Interconect SPI Data
.equ	ADR_USBD   = 0x3F15		;SL811 USB Data Register
.equ	ADR_USBA   = 0x3F16		;SL811 USB Address Register
.equ	ADR_DMAC   = 0x3F17		;DMA Controll register


.equ	ADR_IOP0   = 0x3F20		;AVR IO Data Byte
.equ	ADR_IOP1   = 0x3F21
.equ	ADR_IOP2   = 0x3F22
.equ	ADR_IOP3   = 0x3F23
.equ	ADR_IOP4   = 0x3F24
.equ	ADR_IOP5   = 0x3F25

.equ	ADR_IOD0   = 0x3F26		;AVR IO Direction Control Byte
.equ	ADR_IOD1   = 0x3F27
.equ	ADR_IOD2   = 0x3F28
.equ	ADR_IOD3   = 0x3F29
.equ	ADR_IOD4   = 0x3F2A
.equ	ADR_IOD5   = 0x3F2B


.equ	ADR_IOMA   = 0x3F2C		;IO Interrupt Mask
.equ	ADR_IOMB   = 0x3F2D

.equ	ADR_IOIA   = 0x3F2E		;IO Interrupt Status
.equ	ADR_IOIB   = 0x3F2F




.equ	SRAMBASE  = 0b00000000
.equ	VIDEOBASE = 0b00010000



;***************************************************************************
;**** VARIABLES
.DSEG
.org 0x0100

SCNT1:	 	.BYTE 2		;16 bit Software Timer1 /LED villogas periodusideje
SCNT2:	 	.BYTE 2		;16 bit Software Timer2 /XMODEM hasznalja
SCNT3:		.byte 2		;16 bit SW timer // Beep hasznalja
SCNT4:		.byte 2		;16 bit SW timer // LCD Back Light Timer
SWTmr0:	 	.BYTE 1		;8 bites Software Timer

Updating:	.byte 1		;FLASH update flag

SysFlag:	.byte 1		;System Flag
				;0 = 1 RTC Battery was low on startup
				;1 = 1 RTC not Found

;***************************************************************************
.ESEG


;***************************************************************************
;****		I N T E R R U P T S
;****
;***************************************************************************
.CSEG

	jmp	RESET			; Reset Handler
	jmp	EXT_INT0		; IRQ0 Handler
	jmp	EXT_INT1		; IRQ1 Handler
	jmp	EXT_INT2		; IRQ2 Handler
	jmp	EXT_INT3		; IRQ3 Handler
	jmp	EXT_INT4		; IRQ4 Handler
	jmp	EXT_INT5		; IRQ5 Handler
	jmp	EXT_INT6		; IRQ6 Handler
	jmp	EXT_INT7		; IRQ7 Handler
	jmp	TIM2_COMP		; Timer2 Compare Handler
	jmp	TIM2_OVF		; Timer2 Overflow Handler
	jmp	TIM1_CAPT		; Timer1 Capture Handler
	jmp	TIM1_COMPA		; Timer1 CompareA Handler
	jmp	TIM1_COMPB		; Timer1 CompareB Handler
	jmp	TIM1_OVF		; Timer1 Overflow Handler
	jmp	TIM0_COMP		; Timer0 Compare Handler
	jmp	TIM0_OVF		; Timer0 Overflow Handler
	jmp	SPI_STC			; SPI Transfer Complete Handler
	jmp	USART0_RXC		; USART0 RX Complete Handler
	jmp	USART0_DRE		; USART0,UDR Empty Handler
	jmp	USART0_TXC		; USART0 TX Complete Handler
	jmp	ADC			; ADC Conversion Complete Handler
	jmp	EE_RDY			; EEPROM Ready Handler
	jmp	ANA_COMP		; Analog Comparator Handler
	jmp	TIM1_COMPC		; Timer1 CompareC Handler
	jmp	TIM3_CAPT		; Timer3 Capture Handler
	jmp	TIM3_COMPA		; Timer3 CompareA Handler
	jmp	TIM3_COMPB		; Timer3 CompareB Handler
	jmp	TIM3_COMPC		; Timer3 CompareC Handler
	jmp	TIM3_OVF		; Timer3 Overflow Handler
	jmp	USART1_RXC		; USART1 RX Complete Handler
	jmp	USART1_DRE		; USART1,UDR Empty Handler
	jmp	USART1_TXC		; USART1 TX Complete Handler
	jmp	TWI			; Two-wire Serial Interface Interrupt Handler
	jmp	SPM_RDY			; SPM Ready Handler



.include	"isr.asm"
.include	"M_rs232.asm"
.include	"terminal.asm"
.include	"user.asm"
.include	"xmodem.asm"
.include	"mat32.asm"
.include	"twi.asm"
.include	"RTC8564.asm"
.include	"SmartM.asm"
.include	"usart0.asm"
.include	"gsm.asm"
.include	"sed1375.asm"
.include	"memory.asm"
.include 	"kernel.asm"
.include	"error.asm"

;************************************************************************
;************************************************************************
;* RESET

RESET:		CLI			;GLOBAL INTERRUP DIS.

		ldi R16, high(RAMEND)	;setup stack
		out SPH,R16
		ldi R16, low(RAMEND)
		out SPL,R16


		ldi	R16,0b00001000		;ICNC3 ICES3  WGM33 WGM32 CS32 CS31 CS30
		sts	TCCR3B,R16		;Timer3 Stop

		ldi	R16,0b10000001		;MCUCR enable to change IVSEL
		out	MCUCR,R16
		ldi	R16,0b10100000		;MCUCR, IVSEL point to BOOTSection
		out	MCUCR,R16		;[7] SRE   - 1 External SRAM/XMEM Enable
						;[6] SRW10 - 1 kompatibils modban Wait-state Select Bit
						;[5] SE    - 1 Sleep Enable
						;[4:2] SM1,SM0,SM2- Sleep Mode Select
						;	SM2 SM1 SM0  Sleep Mode
						;	 0   0   0   Idle
						;	 0   0   1   ADC Noise Reduction
						;	 0   1   0   Power-down
						;	 0   1   1   Power-save
						;	 1   0   0   Reserved
						;	 1   0   1   Reserved
						;	 1   1   0   Standby
						;	 1   1   1   Extended Standby
						;[1] IVSEL - 1 Int.Vect. beginning of the Boot Loader section
						;	   - 0 Int.Vect. from 0x0002 beginning of user flash
						;[0] IVCE  - 1 to enable change of the IVSEL bit
						;	   - 0 normal operation

        	ldi	R16,0b00001010		;[6:4] SRL - Sectro Limits
		sts	XMCRA,R16		;[3:2] SRW0
						;[1] - SRW1 (Low bit in MCUCR6)

		clr	R16
		sts	XMCRB,R16		;Full address on Port C, C pot inaccessable

		sts	SCNT3+0,R16
		sts	SCNT3+1,R16		;Beep duration = 0

		sts	SysFlag,R16

;************************************************************************
;**  Timer3 setup
;
		clr	R16
		sts	TCNT3H,R16
		sts	TCNT3L,R16		;Counter=0
		sts	TCCR3C,R16		;FOC3A FOC3B FOC3C     
		ldi	R16,0b00000000
		sts	TCCR3A,R16		;COM3A1 COM3A0 COM3B1 COM3B0 COM3C1 COM3C0 WGM31 WGM30

		ldi	R16,0b00001100		;ICNC3 ICES3  WGM33 WGM32 CS32 CS31 CS30
		sts	TCCR3B,R16		;SYSCLK/256 == 62500Hz

		ldi	R16,0b00010000	 	;  TICIE3 OCIE3A OCIE3B TOIE3 OCIE3C OCIE1C
		sts	ETIMSK,R16

		ldi	R16,High(SYSCLK/(256*SYSTACK))
		sts	OCR3AH,R16
		ldi	R16,Low(SYSCLK/(256*SYSTACK))
		sts	OCR3AL,R16


       		CBI	LED0_PORT,LED0   		;LED vilagit
       		SBI	LED0_DIR,LED0     		;LED -> Output

		CBI	LED1_PORT,LED1   		;LED vilagit
       		SBI	LED1_DIR,LED1     		;LED -> Output

		call	ADC_Init
		call	TWI_Init
		call	Init_UART1
		call	USART0_Init
		call	Init_Term
		call	GSM_Init
		call	SPIIC_Init
		call	Beep_Init

		SEI		       		;SET GLOBAL INT EN.


		ldi	R16,Low(ECopyr)		;Copyright szoveg
		ldi	R17,High(ECopyr)
		call	SendStrW



		ldi	R16,Low(SerNum)		;Serial Number
		ldi	R17,High(SerNum)
		call	SendStrW


		lds	R0,ADR_SERA
                call	HexToUART
		lds	R0,ADR_SERB
                call	HexToUART
		lds	R0,ADR_SERC
                call	HexToUART
		lds	R0,ADR_SERD
                call	HexToUART
		ldi	R16,CR
		call	SendChrW



;************ RAM TEST
;*


		ldi	R16,Low(RAMT)
		ldi	R17,High(RAMT)
		call	SendStrW

		call	ExtSRAMTest
		brcs	SRERR

		ldi	R16,Low(SROkstr)
		ldi	R17,High(SROkstr)
		call	SendStrW
		rjmp	tova1

SRERR:
		ldi	R16,Low(SRERRstr)
		ldi	R17,High(SRERRstr)
		call	SendStrW
		rjmp	tova1


SROkstr:	.db "Pass.",CR,0,0
SRERRstr:	.db "Error. ",CR,0,0
RAMT:		.db "SRAM test...",0,0
VRAMT:		.db "VRAM test...",0,0


GPSXs:		.db "X pos:",0,0
GPSYs:		.db "Y pos:",0,0
GPSZs:		.db "Z pos:",0,0


tova1:



		call	SPI_init

		call	S1D_Init
			call	PutImg
			call	StartDMA
			call	WaitToDMA

		call	USB_Init




;Searching for RTC

		ldi	R16,Low(200)
		sts	SCNT1+0,R16
		ldi	R16,High(200)
		sts	SCNT1+1,R16		;varakozas max 2sec LED piros
MoreWait:
		lds	R16,SCNT1+0
		lds	R17,SCNT1+1
		or	R16,R17
		breq	RTCNotFound

		sleep
		nop

		call	RTC_BatLow		;RTC Battery Ok?
		breq	MoreWait
		brcc	SendDateTime		;Battery Ok

		ldi	R16,Low(LowBat)		;'LOW BATTERY'
		ldi	R17,High(LowBat)
		call	SendStrW

;		call	RTC_ClearLowBat		;Letoroljuk  a LOW BAT bitet , nem toroljuk, csak ha user beallitja
		lds	R16,SysFlag
		ori	R16,1
		sts	SysFlag,R16		;SYSFLAG = LOW BAT
		rjmp	SendDateTime
RTCNotFound:
		ldi	R16,Low(noRTC)		;'RTC Not Found'
		ldi	R17,High(noRTC)
		call	SendStrW
		lds	R16,SysFlag
		ori	R16,2
		sts	SysFlag,R16		;SYSFLAG = RTC not Found

SendDateTime:
       		SBI	LED0_PORT,LED0   		;LED NEM vilagit
							;ide majd a datum es idokiiratas kell

       		CBI	LED1_PORT,LED1   		;Zold LED vilagit
		call	Init_SMedia			;mindekeppen az INTI_UART utan legyen
       		SBI	LED1_PORT,LED1   		;Zold nem LED vilagit


		call	GPS_Init
		call	GPS_Syncronize

;		rjmp	jolkihagy


		ldi	ZL,low(GPSXs*2)
		ldi	ZH,high(GPSXs*2)
		ldi	R16,GPSXs/32768
		out	RAMPZ,R16
		ldi	YL,low(S1D_DISPLAY_SCANLINE_BYTES*200)
		ldi	YH,high(S1D_DISPLAY_SCANLINE_BYTES*200)
		call	PutStrFLASH

		ldi	ZL,low(GPSYs*2)
		ldi	ZH,high(GPSYs*2)
		ldi	YL,low(S1D_DISPLAY_SCANLINE_BYTES*212)
		ldi	YH,high(S1D_DISPLAY_SCANLINE_BYTES*212)
		call	PutStrFLASH

		ldi	ZL,low(GPSZs*2)
		ldi	ZH,high(GPSZs*2)
		ldi	YL,low(S1D_DISPLAY_SCANLINE_BYTES*224)
		ldi	YH,high(S1D_DISPLAY_SCANLINE_BYTES*224)
		call	PutStrFLASH


;		call	PutPhone

jolkihagy:

		call	StartDMA

		call	Beep			;TEST!!!!!!!!!

		call	Kernel_Init

		call	Mem_Init		;Ezt majd legelore kell tenni



		call	SMFAT_Init	;TEST



;***********************************************************************
;***********************************************************************
;********     M     A      I      N				********
;********							********
;***********************************************************************
Main:
		sleep
		nop
		call	LCD_LightMayOff


		call	Terminal
		call	GSMTerminal
		call	GPSTerminal

			;TEST

		call	ICGetByte
		brcc	Main

		mov	R16,R0
		cpi	R16,TouchPress
		breq	TouchPProc

		cpi	R16,TouchMove
		breq	TouchMProc

		cpi	R16,TouchRelease
		brne	Main

TouchRProc:
		ldi	R16,'R'
		rjmp	TouchCom

TouchMProc:	ldi	R16,'M'
		rjmp	TouchCom

TouchPProc:	ldi	R16,'P'

		push	R16
		call	Beep
		call	LCD_LightMayOn
		pop	R16

TouchCom:
		call	SendChrW
		ldi	R16,':'
		call	SendChrW
					;AVR Interconect SPI IRQ Active
Mx1:		call	ICGetByte
		brcc	Mx1
		push	R0

Mx2:		call	ICGetByte
		brcc	Mx2
		call	HexToUART
		pop	R0
		call	HexToUART

		ldi	R16,32
		call	SendChrW

Mx4:		call	ICGetByte
		brcc	Mx4
		push	R0

Mx5:		call	ICGetByte
		brcc	Mx5
		call	HexToUART
		pop	R0
		call	HexToUART

		ldi	R16,CR
		call	SendChrW


		rjmp	Main




SerNum:		.db	"Serial Number: ",0


;************************************************
; LED villogtato rutin

TogleRedLED:	SBIS	LED0_PORT,LED0	;kihagy, ha LED=1
		 RJMP	Led_be
		CBI	LED0_PORT,LED0	;LED eg
		ret
Led_be:		SBI	LED0_PORT,LED0	;LED nem eg
		ret

TogleGreenLED:	SBIS	LED1_PORT,LED1	;kihagy, ha LED=1
		 RJMP	LedG_be
		CBI	LED1_PORT,LED1	;LED eg
		ret
LedG_be:	SBI	LED1_PORT,LED1	;LED nem eg
		ret


.include "audi.asm"
.include "bmp.asm"
.include "gps.asm"
.include "sl811hs.asm"
.include "dflash.asm"
.include "spiic.asm"
.include "adc.asm"
.include "beep.asm"
.include "graph.asm"
.include "disk.asm"
.include "matek.asm"


;.include "phone.asm"


;.include "points\mpoint.asm"


;***************************************************************************
;**** VARIABLES
.DSEG
LEDPer:		.byte 2		;LED villogasi periodusa 0.01s egysegekben
SWstack:	.byte 256	;256 byte azert kellhet stacknek
MaxUsedRAM:	.BYTE 1		;utolso byte jelzo, csak azert hogy ne lepjuk tul az RAMEND byte-ot




