;***************************************************************************
;
; File Name		:'disk.asm"
; Title			:Disk Information
; Date			:2003.08.02.
; Version		:1.2.0
; Support telephone	:+36-70-333-4034,  old: +36-30-9541-658 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:AVR
;
;***************************************************************************
;	D E S C R I P T I O N
;
; Base on:
; Microsoft Extensible Firmware Initiative FAT32 File System Specification
; Hardware White Paper Version 1.03, December 6, 2000
;
;
; Master Boot Record Layout.
;
;Up to offset 1BEh the MBR consists purely of machine code and data (strings etc.).
;At offset 1BEh the first primary partition is defined, this takes 16 bytes,
;after which the second primary partition is defined, followed by the third and fourth,
;the data structures are the same.
;
;Offset		Length		Field
;		(in bytes)
;------------------------------------------
;0x000		0x1BE		Machine Code and Associated Data.
;0x1BE		0x010		Partition 1 Information (see below).
;0x1CE		0x010		Partition 2 Information (see below).
;0x1DE		0x010		Partition 3 Information (see below).
;0x1EE		0x010		Partition 4 Information (see below).
;0x1FE		0x002		Boot Signature AA55h.
;
;
;
;Format of partition information blocks.
;Offset		Length		Field
;		(in bytes)
;------------------------------------------
;00h		1		Set to 80h if this partition is active.
;01h		1		Partition's starting head.
;02h		2		Partition's starting sector and track (see below).
;04h		1		Partition's ID number (see below).
;05h		1		Partition's ending head.
;06h		2		Partition's ending sector and track (see below).
;08h		4		Starting LBA.
;0Ch		4		Partition's length in sectors.;
;
;Format of sector and track information.
; Bits 15-6	Bits 5-0
; Track		Sector
;
;CHS => LBA Converter
;LBA Mode Value = ( C x HpC + H) x SpH + S - 1
; where C is Cylinder Number(starting with 0) /Cylinder == Track
; H is Head Number(starting with 0)
; S is Sector Number(starting with 1)
; HpC is Head per Cylinder
; SpH is Sector per Head


;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  	why
;	----	----------  ---		------------------------------------
;	0.01	2003.08.02  VFX		Creation
;	1.20	2004.03.21  VFX		Redesign to use long file name
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	SYSCLK: f=16.000 MHz (T= 62.5 ns)
;*
;***************************************************************************
;
;

.equ	MSDOSFSROOT	= 0		;cluster 0 means the root dir
.equ	CLUST_FREE	= 0		;cluster 0 also means a free cluster
.equ	MSDOSFSFREE	= CLUST_FREE
.equ	CLUST_FIRST	= 2	    	;first legal cluster number
.equ	CLUST_RSRVD	= 0xfffffff6	;reserved cluster range
.equ	CLUST_BAD	= 0xfffffff7	;a cluster with a defect
.equ	CLUST_EOFS	= 0xfffffff8	;start of eof cluster range
.equ	CLUST_EOFE	= 0xffffffff	;end of eof cluster range

.equ	FAT12_MASK	= 0x00000fff	;mask for 12 bit cluster numbers
.equ	FAT16_MASK	= 0x0000ffff	;mask for 16 bit cluster numbers
.equ	FAT32_MASK	= 0x0fffffff	;mask for FAT32 cluster numbers


;Master Boot Sectror
;
.equ	BootCodeAndData	= 0x0000	;Len: 0x01BE
.equ	Part1Info	= 0x01BE	;Len: 0x0010 Partition 1 Information
.equ	Part2Info	= 0x01CE	;Len: 0x0010 Partition 1 Information
.equ	Part3Info	= 0x01EE	;Len: 0x0010 Partition 1 Information
.equ	Part4Info	= 0x01FE	;Len: 0x0010 Partition 1 Information
.equ	BootSignature	= 0x01FE	;Len: 2 Value: 0xAA55
.equ	SizeofPARTITION = 0x10		;Length of Single Partition record

;Partition table record
;
.equ PR_Is_Active 	= 0x00		;1 Set to 80h if this partition is active.
.equ PR_StartHead	= 0x01		;1 Partition's starting head.
.equ PR_StartSecTrack	= 0x02		;2 Partition's starting sector and track (cylinder)
.equ PR_IDNum		= 0x04		;1 Partition's ID number (Type)
.equ PR_EndHead		= 0x05		;1 Partition's ending head.
.equ PR_EndSecTrack	= 0x06		;2 Partition's ending sector and track (cylinder)
.equ PR_StartLBA	= 0x08		;4 Starting LBA.
.equ PR_Size		= 0x0C		;4 Partition's length in sectors (Size)

;ID numbers, Partition Table Types
;Many of these partition types I have just included because they are listed,
;I don't have any experience with most of them , so be warned I don't know
;that all these partitions follow the same rules as stated before.
;
.equ  PART_ENTRY_UNUSED		= 0x00	;Unused.
.equ  PART_FAT_12		= 0x01	;DOS FAT12 (0-15MB)
.equ  PART_XENIX_1		= 0x02
.equ  PART_XENIX_2		= 0x03	;XENIX /usr
.equ  PART_FAT_16		= 0x04	;DOS FAT16 (16-32MB)
.equ  PART_EXTENDED		= 0x05	;DOS 3.3+ extended partition.
.equ  PART_HUGE			= 0x06	;DOS 3.31+ FAT16 (32MB-2GB)
.equ  PART_IFS			= 0x07	;HPFS / NTFS
.equ  PART_AIX			= 0x08	;AIX boot or SplitDrive
.equ  PART_AIXBootable		= 0x09	;AIX data or Coherent
.equ  PART_OS2BootMgr		= 0x0A	;OS/2 Boot Manager
.equ  PART_FAT32		= 0x0B	;Win95 OSR2+ FAT32 (512MB-2TB)
.equ  PART_FAT32_XINT13		= 0x0C	;PTWin95FAT32LBA, Win95 OSR2+ FAT32 (512MB-2TB LBA).
.equ  PART_XINT13		= 0x0E	;PTWin95FAT16LBA, Win95+ FAT16 (32MB-2GB) (LBA)
.equ  PART_XINT13_EXTENDED	= 0x0F	;PTWin95ExtendedLBA, Win95+ extended partition (LBA)
;				= 0x10	;OPUS
;				= 0x11	;Hidden DOS FAT12
;				= 0x12	;Compaq diagnostics
;				= 0x14	;OS/2 Boot Manager hidden sub-32MB FAT16 partition / Hidden DOS FAT16
;				= 0x16	;OS/2 Boot Manager hidden over-32MB FAT16 partition / Hidden DOS FAT16 (big)
;				= 0x17	;Hidden HPFS/NTFS
;				= 0x18	;AST Windows swapfile
;				= 0x1B	;Hidden Windows95 FAT32 partition
;				= 0x1C	;Hidden Windows95 FAT32 partition (LBA)
;				= 0x1E	;Hidden LBA VFAT partition
;				= 0x24	;NEC DOS
;				= 0x3C	;PartitionMagic recovery
.equ  PART_Venix286		= 0x40	;Venix 80286
.equ  PART_PREP			= 0x41	;Linux/MINIX (sharing disk with DRDOS)
.equ  PART_LDM			= 0x42	;SFS or Linux swap (sharing disk with DRDOS)
;				= 0x43	;Linux native (sharing disk with DRDOS)
;				= 0x50	;DM (disk manager)
.equ  PART_Novell		= 0x51	;DM6 Aux1 (or Novell)
.equ  PART_Microport		= 0x52	;CP/M or Microsoft SysV/AT
;				= 0x53	;DM6 Aux3
;				= 0x54	;DM6
;				= 0x55	;EZ-Drive (disk manager)
;				= 0x56	;Golden Bow (disk manager)
;				= 0x5C	;Priam Edisk (disk manager)
;				= 0x61	;SpeedStor
.equ  PART_UNIX			= 0x63	;GNU Hurd or Mach or Sys V/386 (such as ISC UNIX)(14)
.equ  PART_Netware286		= 0x64	;Novell Netware 286
.equ  PART_Netware386		= 0x65	;Novell Netware 386
;				= 0x70	;DiskSecure Multi-Boot
.equ  PART_PCIX			= 0x75	;PC/IX
;				= 0x77	;QNX4.x
;				= 0x78	;QNX4.x 2nd part
;				= 0x79	;QNX4.x 3rd part
.equ  PART_OldMinix		= 0x80	;MINIX until 1.4a
.equ  PART_Minix		= 0x81	;MINIX / old Linux
.equ  PART_LinuxSwap		= 0x82	;Linux swap
.equ  PART_LinuxExt2		= 0x83	;Linux native(15)
;				= 0x84	;OS/2 hidden C: drive
;				= 0x85	;Linux extended
;				= 0x86	;NTFS volume set
;				= 0x87	;NTFS volume set
.equ  PART_Amoeba		= 0x93
.equ  PART_AmoebaBBT		= 0x94
;				= 0xA0	;IBM Thinkpad hibernation
.equ  PART_BSD			= 0xA5	;BSD/386
;				= 0xA7	;NeXTSTEP 486
.equ  PART_BSDIFS		= 0xB7
.equ  PART_BSDISwap		= 0xB8
;				= 0xC1	;DR DOS 6.0 LOGIN.EXE-secured FAT12 partition
;				= 0xC4	;DR DOS 6.0 LOGIN.EXE-secured FAT16 partition  < 32Mb
;				= 0xC6	;DR DOS 6.0 LOGIN.EXE-secured FAT16 partition >= 32M
.equ  PART_Syrinx		= 0xC7
;			        = 0xCB	;Reserved for DR-DOS secured FAT32
;			        = 0xCC	;Reserved for DR-DOS secured FAT32 (LBA)
;			        = 0xCE	;Reserved for DR-DOS secured FAT16 (LBA)
.equ  PART_MultiDOSFAT12	= 0xD0	;Multiuser DOS secured FAT12
.equ  PART_MultiDOSFAT12s	= 0xD1	;Old Multiuser DOS secured FAT12
.equ  PART_MultiDOSFAT16	= 0xD4	;Old Multiuser DOS secured FAT16 (partitions equal to or smaller than 32MB)
.equ  PART_MultiDOSExt		= 0xD5	;Old Multiuser DOS secured extended partition
.equ  PART_MultiDOSFAT16Ext	= 0xD6	;Old Multiuser DOS secured FAT16 (partitions larger than 32MB)
.equ  PART_CPM			= 0xDB	;CP/M or Concurrent CP/M or Concurrent DOS or CTOS
;equ  PART_DBFS			= 0xE0	;??
.equ  PART_DOSAccess		= 0xE1	;DOS access or SpeedStor 12-bit FAT extended partition
.equ  PART_DOSRO		= 0xE3	;DOS R/O or SpeedStor
.equ  PART_SSFAT16		= 0xE4	;SpeedStor FAT16 extended partition < 1024 cyl.
;				= 0xF1	;SpeedStor
.equ  PART_DOSSecondary		= 0xF2	;DOS 3.3+ secondary
;				= 0xF4	;SpeedStor large partition
;				= 0xFE	;SpeedStor >1024 cyl. or LANstep
.equ  PART_BBT			= 0xFF	;Xenix Bad Block Table


;Format of a boot sector.  This is the first sector of a partition on a hard disk.
;But, it is not the first sector of a partitioned hard disk.
;BootSector33
;
.equ	bs33Jump      		= 0x00		;3 byte jump inst E9xxxx or EBxx90 we needn't it
.equ	bs33OemName		= 0x03		;8 byte OEM name and version
.equ	bs33BPB			= 0x0B		;19 byte BIOS parameter block
.equ	bs33DriveNumber		= 0x1E		;1 byte drive number (0x80 ...)
.equ	bs33BootCode		= 0x1F		;479 byte  Boot Code, we dont use it
.equ	bs33BootSectSig		= 0x1FE		;boot sector signature byte 0x55AA
.equ	BOOTSIG33		= 0x55AA

;bootsector50
.equ	bs50Jump      		= 0x00		;3 byte jump inst E9xxxx or EBxx90 we needn't it
.equ	bs50emName		= 0x03		;8 byte OEM name and version
.equ	bs50BPB			= 0x0B		;25 byte BIOS parameter block
.equ	bs50Ext			= 0x24		;26 byte Bootsector Extension
.equ	bs50BootCode		= 0x3E		;448 byte Boot Code
.equ	bs50BootSectSig		= 0x1FE		;boot sector signature byte 0x55AA
.equ	BOOTSIG50		= 0x55AA

;bootsector710
.equ	bs710Jump      		= 0x00		;3 byte jump inst E9xxxx or EBxx90 we needn't it
.equ	bs710emName		= 0x03		;8 byte OEM name and version
.equ	bs710BPB     		= 0x0B		;53 byte BIOS parameter block
.equ	bs710Ext     		= 0x3D		;26 byte Bootsector Extension
.equ	bs710BootCode		= 0x57		;418 byte Boot Code
.equ	bs710BootSectSig23	= 0x1FC		;only defined for FAT32?
.equ	bs710BootSectSig01	= 0x1FE		;boot sector signature byte 0x55AA
.equ   	BOOTSIG710 		= 0x55AA0000


;ExtBoot
.equ   	exDriveNumber		= 0x00		;1 byte drive number (0x80)
.equ   	exReserved1		= 0x01		;1 byte reserved
.equ   	exBootSignature		= 00x2		;1 byte ext. boot signature (0x29)
.equ   	      EXBOOTSIG		= 0x29
.equ   	exVolumeID		= 0x03		;4 byte volume ID number
.equ   	exVolumeLabel		= 0x07		;11 byte volume label
.equ   	exFileSysType		= 0x12		;8 byte fs type (FAT12 or FAT16)




;BIOS Parameter Block (BPB) for DOS 3.3, 5.0 & 7.10
.equ   	bpbBytesPerSec		= 0x00		;[2] bytes per sector
.equ   	bpbSecPerClust		= 0x02		;[1] sectors per cluster
.equ   	bpbResSectors		= 0x03 		;[2] number of reserved sectors
.equ   	bpbFATs			= 0x05 		;[1] number of FATs
.equ   	bpbRootDirEnts		= 0x06    	;[2] number of root directory entries
.equ   	bpbSectors		= 0x08    	;[2] total number of sectors
.equ   	bpbMedia		= 0x0A    	;[1] media descriptor
.equ   	bpbSectorsPerFAT	= 0x0B    	;[2] number of sectors per FAT
.equ   	bpbSecPerTrack		= 0x0D    	;[2] sectors per track
.equ   	bpbHeads		= 0x0F    	;[2] number of heads
.equ   	bpbHiddenSecs		= 0x11		;[2] number of hidden sectors (or 4 byte >DOS 5.00)
; 3.3 ends here, continue with DOS 5.0
; The difference is bpbHiddenSecs is a short for DOS 3.3,  and bpbHugeSectors is not in the 3.3 bpb.
.equ	bpbHugeSectors		= 0x15		;[4] Large number of sectors if bpbSectors == 0,
						;    Used when volume size is greater than 32 Mb.
;5.0 ends here, continue with DOS 7.10 (FAT32)
;This one has a few extensions to DOS 5.0
.equ	bpbBigFATsecs		= 0x19		;[4] like bpbFATsecs for FAT32
.equ	bpbExtFlags		= 0x1D		;[2] extended flags:
.equ	bpbFSVers		= 0x1F		;[2] filesystem version
.equ	bpbRootClust		= 0x21		;[4] start cluster for root directory
.equ	bpbFSInfo		= 0x25		;[2] filesystem info structure sector
.equ	bpbBackup		= 0x27		;[2] backup boot sector



; struct _DISK_GEOMETRY
; DISK_GEOMETRY
.equ	DiskNumber		= 0x00	;[1]  0x80 for C, 0x81 for D ect.
.equ	MediaType		= 0x01	;[1]  MEDIA_TYPE -> Byte
.equ	MediaFlag		= 0x02	;[1]
.equ	Cylinders		= 0x03	;[2]
.equ	Heads			= 0x05	;[1]
.equ	Sectros			= 0x06	;[1]
.equ	SectorsPerTrack		= 0x07	;[1]
.equ	SectorPerCluster 	= 0x08	;[1]
.equ	CountOfClusters		= 0x09	;[4]
.equ	MaxSector		= 0x0D	;[4]
.equ	BytesPerSector		= 0x11	;[2]  <- BytesPerSector
.equ	FirstSector		= 0x13	;[4]
.equ	LastSector		= 0x17	;[4]
.equ	FirstDataSector		= 0x1B	;[4]
.equ	FirstRootSector		= 0x1F	;[4]
.equ	FirstFatSector		= 0x23	;[4]
.equ	FATSz			= 0x27	;[4]
.equ	FAT32RootCluster	= 0x2B	;[4]
.equ	RootDirSectors		= 0x2F	;[4]
.equ	FATtype	     		= 0x33	;[1]

.equ	SizeOfDgeo		= 0x34	;--  size of this record


.equ	FixedMedia	= 0x01
.equ	RemovableMedia	= 0x02


.equ	FAT12	= 12
.equ	FAT16	= 16
.equ	FAT32	= 32


;The regular DOS File Directory Entry Area: (FDE Area)
;The Root Directory contains 32-byte FDE's. These 32 byte entries contain information about the file:
;File Name, attribute, time, date, etc.
;Here is the format of a regular DOS File Directory Entry:

;struct direntry
.equ	deName		= 0x00			;[8]	filename, blank filled
.equ	deExtension	= 0x08			;[3]	extension, blank filled
.equ	deAttributes	= 0x0B			;[1]	file attributes
.equ	deLowerCase	= 0x0C			;[1]	NT VFAT lower case flags
.equ	deCTime_ms	= 0x0D			;[1]	hundredth of seconds in CTime
.equ	deCTime		= 0x0E			;[2]	create time
.equ	deCDate		= 0x10			;[2]	create date
.equ	deADate		= 0x12			;[2]	access date
.equ	deHighClust	= 0x14			;[2]	high bytes of starting cluster number (only in FAT32)
.equ	deMTime		= 0x16			;[2]	last update time / time stamp
.equ	deMDate		= 0x18			;[2]	last update date / date stamp
.equ	deStartCluster	= 0x1A			;[2]	starting cluster of file
.equ	deFileSize	= 0x1C			;[4]	size of file in bytes


;Directory first char:
.equ	SLOT_EMPTY     		= 0x00	;slot has never been used
.equ	SLOT_E5	      		= 0x05	;the real value is 0xe5
.equ	SLOT_DELETED		= 0xe5	;file in this slot deleted

;directory attribute
.equ	ATTR_NORMAL		= 0x00	;normal file
.equ	ATTR_READONLY		= 0x01	;file is readonly
.equ	ATTR_HIDDEN    		= 0x02	;file is hidden
.equ	ATTR_SYSTEM    		= 0x04	;file is a system file
.equ	ATTR_VOLUME    		= 0x08	;entry is a volume label
.equ	ATTR_LONG_FILENAME	= 0x0f	;this is a long filename entry
.equ	ATTR_DIRECTORY		= 0x10	;entry is a directory name
.equ	ATTR_ARCHIVE		= 0x20	;file is new or modified

.equ	LCASE_BASE		= 0x08	;filename base in lower case
.equ	LCASE_EXT		= 0x10	;filename extension in lower case



; number of directory entries in one sector (512 byte/sector)
.equ	DIRENTRIES_PER_SECTOR	= 0x10

;Structure of a Win9x long name directory entry ( LFN File Directory Entry)
;struct winentry
.equ	weCnt			= 0x00	;[1]	LFN record sequence and flags byte
.equ	wePart1			= 0x01	;[10]	long file name (first part, 5 char in file name = 10 byte; 16 bit Unicode)
.equ	weAttributes		= 0x0B	;[1]    Attributes (0Fh)
.equ	weReserved		= 0x0C	;[1]    reserved
.equ	weChksum		= 0x0D	;[1]    Check sum for short file name (8.3 alias)
.equ	wePart2			= 0x0E	;[12]   long file name (second part, 6 char) (16 bit Unicode)
.equ	weReserved2		= 0x1A	;[2]    starting cluster number (always 0000h for LFN records)
.equ	wePart3			= 0x1C	;[4]    long file name (third part, 2 char) (16 bit Unicode)

;First byte (weCnt):
;This is the LFN slot number in the wanted sequence of slots for this LFN.
;Bits 5-0 are the sequence number with the last sequence number OR'd with 40h.
;These LFN slots are directly behind the regular short file name File Directory Entry.
;A note : Bit 6 of above denotes last LFN slot for file (i.e: OR'd with 40h above),
;and Bit 7 set if file deleted. Slots are numbered from 1 to N

.equ	WIN_CNT			= 0x3f
.equ	WIN_BitLast		= 6
.equ	WIN_BitDeleted		= 7

.equ	WIN_CHARS		= 13	;Number of chars per winentry

;Maximum filename length in Win95 Note: Must be < sizeof(dirent.d_name)
.equ	WIN_MAXLEN		= 255

;LFN of Thisisalongfilename.andextension file
;Please note that the LFN FDE's are in reverse order.
;Meaning that the first part of the LFN is in the FDE in slot 3,
;with the second part of the LFN in slot 2,
;and the last LFN FDE needed in slot 1 with its first byte OR'd with 40h.
;So the first byte in the FDE in slot 3 would be 01h, with the first byte in slot 2,
;being 02h, and the first byte in slot 1, being 43h.
;
;With all of this in mind, the layout of the four FDE's would be:
;
;other entries....
;slot #1, id = 43h, characters = "ension"
;slot #2, id = 02h, characters = "lename.andext"
;slot #3, id = 01h, characters = "Thisisalongfi"
;slot #4, regular short filename Directory entry
;other entries....
;


;**************************************************************************
;* Hardware Def.
;


;***************************************************************************
;**** VARIABLES
.DSEG

;SmartMedia HDD1
HDD1Handle:	.byte 3
HDD1Process:	.byte 2

;***************************************************************************
.ESEG


;***************************************************************************
;**** CODE SEG
;***************************************************************************
.CSEG


;FAT structures initialization for SmartMedia
SMFAT_Init:
		call	CreateNewProcessId 			;we get a new process ID in R17:R16
		mov	R19,R16
		mov	R20,R17
		sts	HDD1Process+0,R16
		sts	HDD1Process+1,R17			;Save new Process Number
		clr	R18
		ldi	R17,high(SizeOfDgeo+512)
		ldi	R16,low(SizeOfDgeo+512)
								;need 512 byte + size of disk geomatery
		call	MmAllocMemory	      			;R18:R17:16 - phisical Memory pointer (MCB+MCBLen)
		brcs  	HDD1_Error1	     			;c= 1 error, R17:R16 - error code
		sts	HDD1Handle+0,R16
		sts	HDD1Handle+1,R17
		sts	HDD1Handle+2,R18
		call	MmSetPage				;Sets Z to allocated memory

		call	SM_ResetDevice
		brcs	HDD1_Error0
		clr	R10
		clr	R11
		clr	R12					;Read Sector 0 for Partition Table
		ldi	XL,low(512)	     			;X - count, Y - memory pointer
		ldi	XH,high(512)
		movw	YL,ZL
		call	SM_SearchAndReadSector
		brcc	HDD1_Noerr1
HDD1_Error0:
		rjmp	HDD1_ErrorRealiseMem
HDD1_Error1:
		rjmp	HDD1_Error
HDD1_Noerr1:
		ldi	YL,low(Part1Info)    			;address of 1st partition
		ldi	YH,high(Part1Info)
		add	YL,ZL
		adc	YH,ZH		    			;X -> 1st partition
					     			;Z -> Disk geometry

		ldi	R20,FAT12
		ldd	R16,Y+PR_IDNum	     			;Partition type, we support only FAT12, FAT16 and FAT32
		cpi	R16,PART_FAT_12				;DOS 12-bit FAT
		breq	FSOk
		ldi	R20,FAT16
		cpi	R16,PART_XINT13				;Win95 FAT16
		breq	FSOk
		cpi	R16,PART_XINT13_EXTENDED		;Win95 FAT16 (LBA)
		breq	FSOk
		cpi	R16,PART_FAT_16				;DOS 16-bit <32M
		breq	FSOk
		cpi	R16,PART_EXTENDED			;DOS Extended
		breq	FSOk
		cpi	R16,PART_HUGE				;DOS 16-bit >=32M
		breq	FSOk
		ldi	R20,FAT32
		cpi	R16,PART_FAT32				;Win95 FAT32
		breq	FSOk
		cpi	R16,PART_FAT32_XINT13			;Win95 FAT32 (LBA)
		breq	FSOk
		ldi	R16,low(ERROR_PARTITION_FAILURE)
		ldi	R17,high(ERROR_PARTITION_FAILURE)
		rjmp	HDD1_ErrorRealiseMem
FSOk:
								;Partition Ok.
		std	Z+FATtype,R20
		ldi	R16,0x81	     			;This will Drive D:
		std	Z+DiskNumber,R16
		ldi	R16,RemovableMedia
		std	Z+MediaType,R16	     			;MEDIA_TYPE -> Removable
		clr	R16
		std	Z+MediaFlag,R16

		lds	R16,SmPages+0				;Max Sector Number
		std	Z+MaxSector+0,R16
		lds	R16,SmPages+1
		std	Z+MaxSector+1,R16
		lds	R16,SmPages+2
		std	Z+MaxSector+2,R16
		clr	R16
		std	Z+MaxSector+3,R16

		lds	R16,SmPPB
		std	Z+SectorPerCluster,R16

		lds	R16,SmBlocks+0
		std	Z+CountOfClusters+0,R16
		lds	R16,SmBlocks+1
		std	Z+CountOfClusters+1,R16
		clr	R16
		std	Z+CountOfClusters+2,R16
		std	Z+CountOfClusters+3,R16

		ldi	R16,Low(512)
		std	Z+BytesPerSector+0,R16
		ldi	R16,High(512)
		std	Z+BytesPerSector+1,R16

		ldd	R10,Y+PR_StartLBA+0			;Starting LBA.
		std	Z+FirstSector+0,R10
		ldd	R11,Y+PR_StartLBA+1
		std	Z+FirstSector+1,R11
		ldd	R12,Y+PR_StartLBA+2
		std	Z+FirstSector+2,R12
		ldd	R16,Y+PR_StartLBA+3
		std	Z+FirstSector+3,R16

		ldd	R16,Y+PR_Size+0				;Partition's length in sectors (Size of Partition)
		std	Z+LastSector+0,R16
		ldd	R16,Y+PR_Size+1
		std	Z+LastSector+1,R16
		ldd	R16,Y+PR_Size+2
		std	Z+LastSector+2,R16
		ldd	R16,Y+PR_Size+3
		std	Z+LastSector+3,R16
								;Read BIOS Parameter Block
		ldi	YL,low(SizeOfDgeo)			;address of BIOS Parameter Block
		ldi	YH,high(SizeOfDgeo)
		add	YL,ZL
		adc	YH,ZH		    			;X -> 1st partition
		ldd	XL,Z+BytesPerSector+0			;Read 1 sector at R12:R11:R10
		ldd	XH,Z+BytesPerSector+1
		call	SM_SearchAndReadSector
		brcc	HDD1_Noerr2
		rjmp	HDD1_ErrorRealiseMem			;Error, free allocated memory
HDD1_Noerr2:
					     			;Process BIOS Parameter Block
		ldi	YL,Low(SizeOfDgeo+bs710BPB)
		ldi	YH,High(SizeOfDgeo+bs710BPB)
		add	YL,ZL
		adc	YH,ZH					; Y --> BPB
						;FirstFATSector	= bpb->bpbResSectors + first_sec;
		ldd	R16,Y+bpbResSectors+0
		ldd	R17,Y+bpbResSectors+1
		clr	R18
		ldd	R0,Z+FirstSector+0
		ldd	R1,Z+FirstSector+1
		ldd	R2,Z+FirstSector+2
		ldd	R3,Z+FirstSector+3
		add	R0,R16
		adc	R1,R17
		adc	R2,R18
		adc	R3,R18
		std	Z+FirstFatSector+0,R0
		std	Z+FirstFatSector+1,R1
		std	Z+FirstFatSector+2,R2
		std	Z+FirstFatSector+3,R3

		ldd	R2,Y+bpbFATs				;Number of FAT copy
		clr	R3
		clr	R4
		clr	R5

		ldd	R16,Z+FATtype
		cpi	R16,FAT32
		brne	SmallerFAT
		rjmp	SmallerFAT		;!!!!!!! TEST !!!!!!!

								;FAT32 here
		ldd	R21,Y+bpbBigFATsecs+0   		;Size of FAT in sectors
		ldd	R22,Y+bpbBigFATsecs+1
		ldd	R23,Y+bpbBigFATsecs+2
		ldd	R24,Y+bpbBigFATsecs+3
		call	mult32					;R24:R21 x R5:R2 -> R15:R8

		std	Z+FATSz+0,R8				;Size Of FAT in sectors
		std	Z+FATSz+1,R9
		std	Z+FATSz+2,R10
		std	Z+FATSz+3,R11

		ldd	R16,Y+bpbRootClust+0			;FirstDirCuster	= bpb->bpbRootClust;
		std	Z+FAT32RootCluster+0,R16
		ldd	R16,Y+bpbRootClust+1
		std	Z+FAT32RootCluster+1,R16
		ldd	R16,Y+bpbRootClust+2
		std	Z+FAT32RootCluster+2,R16
		ldd	R16,Y+bpbRootClust+3
		std	Z+FAT32RootCluster+3,R16
	 					;FirstDataSectorD = bpb->bpbResSectors + FATSz + first_sec;
;		rjmp	FATx01

SmallerFAT:
								;FAT12 & FAT16 here
		ldd	R21,Y+bpbSectorsPerFAT+0   		;Size of FAT in sectors
		ldd	R22,Y+bpbSectorsPerFAT+1
		clr	R23
		clr	R24
		call	mult32					;R24:R21 x R5:R2 -> R15:R8

		std	Z+FATSz+0,R8
		std	Z+FATSz+1,R9
		std	Z+FATSz+2,R10
		std	Z+FATSz+3,R11				;Size Of FAT in sectors
	       					;FirstDirSector = FirstFATSector + FATSz;
		ldd	R0,Z+FirstFatSector+0
		ldd	R1,Z+FirstFatSector+1
		ldd	R2,Z+FirstFatSector+2
		ldd	R3,Z+FirstFatSector+3
		add	R0,R8
		adc	R1,R9
		adc	R2,R10
		adc	R3,R11
		std	Z+FirstRootSector+0,R0
		std	Z+FirstRootSector+1,R1
		std	Z+FirstRootSector+2,R2
		std	Z+FirstRootSector+3,R3

;	FirstDataSectorD = bpb->FirstDirSector + (bpb->bpbRootDirEnts / 16)

		ldd	R16,Y+bpbRootDirEnts+0
		ldd	R17,Y+bpbRootDirEnts+1
		lsr	R17  				;bpbRootDirEnts / 16
		ror	R16
		lsr	R17
		ror	R16
		lsr	R17
		ror	R16
		lsr	R17
		ror	R16
		clr	R18
		add	R0,R16
		adc	R1,R17
		adc	R2,R18
		adc	R3,R18
		std	Z+FirstDataSector+0,R0
		std	Z+FirstDataSector+1,R1
		std	Z+FirstDataSector+2,R2
		std	Z+FirstDataSector+3,R3




;debug print
		rjmp	kihagy1

		ldd	R10,Z+FirstRootSector+0			;start Sector
		ldd	R11,Z+FirstRootSector+1
		ldd	R12,Z+FirstRootSector+2
		ldd	R13,Z+FirstRootSector+3
		ldd	XL,Z+BytesPerSector+0			;X - count
		ldd	XH,Z+BytesPerSector+1
		ldi	YL,high(SizeOfDgeo)			;Y - memory pointer
		ldi	YH,low(SizeOfDgeo)
		add	YL,ZL
		adc	YH,ZH
		call	SM_SearchAndReadSector
		brcc	HDD1_Noerr3
		rjmp	HDD1_Error
HDD1_Noerr3:
		ldi	YL,high(SizeOfDgeo)
		ldi	YH,low(SizeOfDgeo)
		add	YL,ZL
		adc	YH,ZH
		ldi	XL,low(256)
		ldi	XH,high(256)
inl:		push	XL
		push	XH
		ld	R16,Y+
		push	YL
		push	YH
		call	SendChrW
		pop	YH
		pop	YL
		pop	XH
		pop	XL
		sbiw	XL,1
		brne	inl

kihagy1:

		ldi	R16,CR
		call	SendChrW


		ldi	R16,Low(suce)
		ldi	R17,High(suce)
		call	SendStrW
		ret


suce:		.db	"Mounting Drive D: Sucessfull",CR,0

HDD1_ErrorRealiseMem:
		lds	R16,HDD1Handle+0		;Free Allocated Memory
		lds	R17,HDD1Handle+1
		lds	R18,HDD1Handle+2
		lds	R19,HDD1Process+0
		lds	R20,HDD1Process+1
		call	MmReleaseMemory
HDD1_Error:
		call	DBGErrorCode
		sec
		ret


;*************************************************************************
; Calculating the Check Sum of LFN Dir Entry
; In: X - Pointer to File name entry
; Out: R16 - checksum of short File name
;
LFNCheckSum:
		ldi	R17,10		;Length of entry - 1 
		ld	R16,X+
LFN01:		bst	R16,0
		ror	R16
		bld	R16,7		;ror and R16 bit 0 store in bit 7
		ld	R0,X+
		add	R16,R0
		dec	R17
		brne	LFN01
		ret

;*******************************************
;* Convert CHS to LBA
;*
; CHS Mode : Refers to Cylinder,Head,Sector Mode.
; LBA Mode : Refers to Logical Block Address indicating sequential
; (0, 1, 2, 3,.) Counting Value of CHS.
;
;In:
;   R16 - Drive number C: 0x80, D: 0x81 ....
;   R18:R17 - Cylinder number (starting with 0)
;   R19 - Head number (starting with 0)
;   R20 - Sector number (starting with 1)
;
;Out:
;   (R19:R18:) R17:R16 - LBA    (currently 16 bit LBA only)
;

CHStoLBA:




		ret
