;***************************************************************************
;
; File Name		:'graph.asm"
; Title			:Grafikai rutinok 
; Date			:2003.08.09.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034,  old: +36-30-9541-658 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:AVR
;
;***************************************************************************
;	D E S C R I P T I O N
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  	why
;	----	----------  ---		------------------------------------
;	0.01	2003.08.09  VFX		Creation
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	SYSCLK: f=16.000 MHz (T= 62.5 ns)
;*
;***************************************************************************
;
; Vonal huzas
;http://www.cs.helsinki.fi/group/goa/mallinnus/lines/bresenh.html
;http://www.gamedev.net/reference/articles/article1275.asp
;http://www.edepot.com/linee.html
;http://graphics.lcs.mit.edu/~mcmillan/comp136/Lecture6/Lines.html
;
;
;**************************************************************************
;* Hardware Def.
;
;***************************************************************************
;**** VARIABLES
.DSEG
lastx:	.byte 2		;utolso X koordinata
lasty:	.byte 2		;utolso Y koordinata
lastc:	.byte 1		;utolso Color 


;***************************************************************************
.ESEG


;***************************************************************************
;**** CODE SEG
;***************************************************************************
.CSEG


;*-------------------------------------------------------------------------
;** seSetLcdPixel(x,y, color) , seSetCrtPixel(x, y, color)
;**
;**    In:	R17:R16 - X
;**		R19:R18 - Y
;**		R20 - Color
;**		XLH, XHL, XHH lapokra legyen belapozva a 48kb VIDEO MEMORIA
;**    Out:	- 
;*
seSetPixel:
		sts	lastx+0,R16
		sts	lastx+1,R17
		sts	lasty+0,R18
		sts	lasty+1,R19
		sts	lastc+0,R20
		ldi	R25,high(S1D_DISPLAY_WIDTH)
		cpi	R16,low(S1D_DISPLAY_WIDTH)
		cpc	R17,R25
		brsh	seNoPixel
		ldi	R25,high(S1D_DISPLAY_HEIGHT)
		cpi	R18,low(S1D_DISPLAY_HEIGHT)
		cpc	R19,R25
		brsh	seNoPixel
		
		ldi	ZL,low(MMAP1)
		ldi	ZH,high(MMAP1)
		ldi	R25,S1D_DISPLAY_SCANLINE_BYTES
		mul	R25,R18
		movw	R24,R16
		lsr	R25
		ror	R24
		brcc   leftpixel
		rjmp	rightpixel
seNoPixel:	ret

rightpixel:
		add	R0,R24
		adc	R1,R25
		add	ZL,R0
		adc	ZH,R1
		ld	R25,Z
		andi	R25,0xF0
		mov	R24,R20
		andi	R24,0x0F
		or	R25,R24
		st	Z,R25
		ret

leftpixel:
		add	R0,R24
		adc	R1,R25
		add	ZL,R0
		adc	ZH,R1
		ld	R25,Z
		andi	R25,0x0F
		mov	R24,R20
		swap	R24
		andi	R24,0xF0
		or	R25,R24
		st	Z,R25
		ret



;**************************************
;** seDrawLine(x1, y1, x2, y2, color)
;**  IN:  R17:R16 - x1
;**       R19:R18 - y1
;**	  R21:R20 - x2
;**	  R23:R22 - y2
;**	  R24 - color
;**  OUT: -
;*
seDrawLine:
		ldi	R25,SRAMBASE
		sts	ADR_EXLH,R25
		inc	R25
		sts	ADR_EXHL,R25
		inc	R25
		sts	ADR_EXHH,R25

				;if (x2 >= x1) then {xinc1 = 1; xinc2 = 1}
				;	       else {xinc1 =-1; xinc2 =-1}
				;xinc1 - R15:R14
				;xinc2 - R13:R12
		clr	R15
		clr	R14
		clr	R13
		clr	R12
		cp	R20,R16
		cpc	R21,R17
		brsh	seDL01
		dec	R15
		dec	R14
		dec	R13
		dec	R12
		rjmp	seDL02		 
seDL01:
		inc	R14
		inc	R12
seDL02:				;if (y2 >= y1) then {yinc1 = 1; yinc2 = 1;}
				;	       else {yinc1 =-1; yinc2 =-1;}
				;yinc1 - R11:R10
				;yinc2 - R9:R8
		clr	R11
		clr	R10
		clr	R9
		clr	R8
		cp	R22,R18
		cpc	R23,R19
		brsh	seDL03
		dec	R11
		dec	R10
		dec	R9
		dec	R8
		rjmp	seDL04		 
seDL03:
		inc	R10
		inc	R8
seDL04:
					;deltax = abs(x2 - x1); The difference between the x's
		sub	R20,R16		;deltax R21:R20
		sbc	R21,R17
		brpl	seDLPlus
		movw	R0,R20
		clr	R20
		clr	R21
		sub	R20,R0
		sbc	R21,R1

seDLPlus:				;deltay = abs(y2 - y1); The difference between the y's
		sub	R22,R18		;deltay R23:R22 
		sbc	R23,R19
		brpl	seDL05

		movw	R0,R22
		clr	R22
		clr	R23
		sub	R22,R0
		sbc	R23,R1

seDL05:					;x - R17:R16
					;Y - R19:R18
					;den - R7:R6
					;num - R5:R4
					;numpixels, curpixels - XH:XL
					;numadd - R3:R2

		cp	R20,R22		;if (deltax >= deltay)
		cpc	R21,R23		;There is at least one x-value for every y-value
		brsh	seDL06
		clr	R13		;xinc2 = 0; Don't change the x for every iteration
		clr	R12
		clr	R11		;yinc1 = 0; Don't change the y when numerator >= denominator
		clr	R10
		movw	R6,R22		;den = deltay;
		movw	R4,R22		;num = deltay / 2;
		lsr	R5
		ror	R4
		movw	R2,R20		;numadd = deltax;
		movw	XL,R22		;numpixels = deltay; There are more y-values than x-values
		rjmp	seDL07


seDL06:
		clr	R15		;xinc1 = 0; Don't change the x when numerator >= denominator
		clr	R14
		clr	R9		;yinc2 = 0; Don't change the y for every iteration
		clr	R8
		movw	R6,R20		;den = deltax;
		movw	R4,R20		;num = deltax / 2;
		lsr	R5
		ror	R4
		movw	R2,R22		;numadd = deltay;
		movw	XL,R20		;numpixels = deltax;  There are more x-values than y-values

seDL07:					;for (curpixel = 0; curpixel <= numpixels; curpixel++)
					;curpixel, numpixel - R23:R22
		mov	R20,R24		;Color
seDL08:					;ciklus kezdete,  PutPixel(x, y); Draw the current pixel
		rcall	seSetPixel
		add	R4,R2		;num += numadd;  Increase the numerator by the top of the fraction
		adc	R5,R3
		cp	R4,R6		;if (num >= den)  Check if numerator >= denominator
		cpc	R5,R7
		brlo	seDL09
		sub	R4,R6
		sbc	R5,R7		;num -= den; Calculate the new numerator value
		add	R16,R14		;x += xinc1; Change the x as appropriate
		adc	R17,R15
		add	R18,R10		;y += yinc1; Change the y as appropriate
		adc	R19,R11
seDL09:
		add	R16,R12		;x += xinc2; Change the x as appropriate
		adc	R17,R13
		add	R18,R8		;y += yinc2; Change the y as appropriate
		adc	R19,R9

		sbiw	XL,1
		brne	seDL08		;curpixel <= numpixels

		ret



;****************************************
; R17:R16 - Left (X)
; R19:R18 - Top (Y)
; R21:R20 - Width
; R23:R22 - Height
;     R24 - Color
;
DrawBox:
		andi	R24,0x0F
		mov	R4,R24
		swap	R24
		or	R4,R24		;2 pixel / byte
	
		ldi	R24,low(S1D_DISPLAY_WIDTH)
		ldi	R25,High(S1D_DISPLAY_WIDTH)
		cp	R16,R24
		cpc	R17,R25
		brsh	NoDrawBox1
		rjmp	DrBox00


NoDrawBox1:	ret

DrBox00:
		movw	R2,R20
		add	R2,R16
		adc	R3,R17
		cp	R2,R24
		cpc	R3,R25
		brlo	DrBox01
		sub	R24,R16
		sbc	R25,R17
		movw	R20,R24

DrBox01:
		ldi	R24,low(S1D_DISPLAY_HEIGHT)
		ldi	R25,High(S1D_DISPLAY_HEIGHT)
		cp	R18,R24
		cpc	R19,R25
		brsh	NoDrawBox
		movw	R2,R22
		add	R2,R18
		adc	R3,R19
		cp	R2,R24
		cpc	R3,R25
		brlo	DrBox02
		sub	R24,R18
		sbc	R25,R19
		movw	R22,R24
DrBox02:
		ldi	R25,SRAMBASE		;VIDEOBASE
		sts	ADR_EXLH,R25
		inc	R25
		sts	ADR_EXHL,R25
		inc	R25
		sts	ADR_EXHH,R25

		ldi	ZL,low(MMAP1)
		ldi	ZH,high(MMAP1)
		ldi	R25,S1D_DISPLAY_SCANLINE_BYTES
		mul	R25,R18
		lsr	R17
		ror	R16
		add	R0,R16
		adc	R1,R17
		add	ZL,R0
		adc	Zh,R1

		lsr	R21
		ror	R20		;lenght in byte
DrBox04:
		push	ZL
		push	ZH
		movw	XL,R20
DrBox05:
		st	Z+,R4
		sbiw	XL,1
		brne	DrBox05
		pop	ZH
		pop	ZL
		ldi	R24,low(S1D_DISPLAY_SCANLINE_BYTES)
		ldi	R25,High(S1D_DISPLAY_SCANLINE_BYTES)
		add	ZL,R24
		adc	ZH,R25
		movw	XL,R22
		sbiw	XL,1
		movw	R22,XL
		brne	DrBox04
NoDrawBox:
		ret
	




;****************************************
; R17:R16 - Left (X)
; R19:R18 - Top (Y)
; R21:R20 - Width
; R23:R22 - Height
;
DrawWinBorder:
		andi	R16,0b11111110
		andi	R20,0b11111110		;paratlan hosszu nem lehet
		clr	R2
		cpi	R20,6
		cpc	R21,R2			;min width > 6
		brsh	DrWinB01
DrWinB00:	sec
		ret

DrWinB01:	cpi	R22,4			;min height>4
		cpc	R23,R2
		brlo	DrWinB00

		ldi	R25,SRAMBASE		;VIDEOBASE
		sts	ADR_EXLH,R25
		inc	R25
		sts	ADR_EXHL,R25
		inc	R25
		sts	ADR_EXHH,R25

		ldi	ZL,low(MMAP1)
		ldi	ZH,high(MMAP1)
		ldi	R25,S1D_DISPLAY_SCANLINE_BYTES
		mul	R25,R18
		movw	R24,R16
		lsr	R25
		ror	R24
		add	R0,R24
		adc	R1,R25
		add	ZL,R0
		adc	Zh,R1

		lsr	R21
		ror	R20		;lenght in byte

		movw	XL,R20
		sbiw	XL,1
		movw	YL,ZL
		ldi	R24,0xBB
DrWinB02:
		st	Y+,R24
		sbiw	XL,1
		brne	DrWinB02
		ldi	R24,0xB0
		st	Y,R24		;elso csik megvolt



		ldi	R24,low(S1D_DISPLAY_SCANLINE_BYTES)
		ldi	R25,High(S1D_DISPLAY_SCANLINE_BYTES)
		movw	R4,R24
		add	ZL,R4
		adc	ZH,R5
		movw	YL,ZL
		ldi	R24,0xBD
		st	Y+,R24
		movw	XL,R20
		ldi	R24,0xDD
		sbiw	XL,2
DrWinB03:
		st	Y+,R24
		sbiw	XL,1
		brne	DrWinB03
		ldi	R24,0x90
		st	Y,R24		;masodik csik meg volt



		movw	XL,R22
		sbiw	XL,4
		ldi	R24,0xBD
		ldi	R25,0x90
DrWinB04:
		add	ZL,R4
		adc	ZH,R5
		movw	YL,ZL
		st	Y,R24
		add	YL,R20
		adc	YH,R21
		sbiw	YL,1		;??
		st	Y,R25
		sbiw	XL,1
		brne	DrWinB04
					;fuggoleges csikok megvoltak

		add	ZL,R4
		adc	ZH,R5
		movw	YL,ZL
		ldi	R24,0xB9
		st	Y+,R24
		movw	XL,R20
		sbiw	XL,2
		ldi	R24,99
DrWinB05:
		st	Y+,R24
		sbiw	XL,1
		brne	DrWinB05
		ldi	R24,0x90
		st	Y,R24		;utolso elotti csik

		add	ZL,R4
		adc	ZH,R5
		movw	YL,ZL
		ldi	R24,0x00
		movw	XL,R20
DrWinB06:
		st	Y+,R24
		sbiw	XL,1
		brne	DrWinB06	;utolso csik

		ret


;****************************************
; R17:R16 - Left (X)
; R19:R18 - Top (Y)
; R21:R20 - Width
; R23:R22 - Height
;     R24 - Color

DrawWin:
		push	R24
		push	R20
		push	R21
		rcall	DrawWinBorder
		pop	R21
		pop	R20
		ldi	R24,2
		clr	R0
		add	R16,R24
		adc	R17,R0
		add	R18,R24
		adc	R19,R0
		ldi	R24,4
		sub	R20,R24
		sbc	R21,R0
		sub	R22,R24
		sbc	R23,R0
		pop	R24
		rcall	DrawBox
		ret


