; ** FAST signed/unsigned 32bit arithmetic Routines for Atmel AVR RISC **
; ** March 1999, by Jack Tidwell <jackt@igalaxy.net> **
; ** These routines may be used for personal and educational
; **      purposes only, and are free to download. **
;

;last mod 2001.12.17.

;.def    PROD    = r8
;.def    QUOT    = R8             ; Quot/Prod LSByte
;.def    PRODM   = r9
;.def    QUOTM   = r9
;.def    PRODH   = r10
;.def    QUOTH   = r10
;.def    PRODHH  = r11
;.def    QUOTHH  = r11            ; Quot/Prod MSByte


;.def    MANT1   = r16            ; ACC1 LSByte
;.def    MANT1m  = r17
;.def    MANT1h  = r18
;.def    MANT1hh = r19            ; ACC1 MSByte

;.def    MANT2   = r20            ; 32bit Multiplier/Divisor
;.def    MANT2m  = r21
;.def    MANT2h  = r22
;.def    MANT2hh = r23
.def    count   = r24            ; bit count
.def    temp    = r25

.dseg
ASCIIBuf: .byte 12              ; Ascii Number buffer.

.cseg
;** Acc1 += Acc2 **
add32:	add	R16,R20
	adc	R17,R21
	adc	R18,R22
	adc	R19,R23
	ret

;** Acc1 -= Acc2 **
SUB32:	sub	R16,R20
	sbc	R17,R21
	sbc	R18,R22
	sbc	R19,R23
	ret

TSTZERO1: clr   temp            ; Test for Acc1 == 0
        or      temp,R16
        or      temp,R17
        or      temp,R18
        or      temp,R19
	ret

TSTZERO2: clr   temp            ; Test for Acc2 == 0
	or	temp,R20
	or	temp,R21
	or	temp,R22
	or	temp,R23
	ret

;;** Acc1 %= Acc2 **
MOD:    rcall   SDIV                    ;* put the remainder into Acc1 *
	mov	R16,R8
	mov	R17,R9
	mov	R18,R10
	mov	R19,R11
	ret

;;** Signed 32bit Division **
SDIV:   tst	R19			; test MSBit (b31) for Sign
	brmi	SDIV1n
	tst	R23
        brpl	UDIV			; both are Pos, normal Division
        rcall   NEGMANT2	        ; Acc1+, Acc2-, Negate Acc2
SDIV1a: rcall   UDIV			; Normal Div
        rjmp    NEGMANT1		; Negate Results

SDIV1n: rcall	NEGMANT1
	tst	R23
	brpl	SDIV1a
                                ;both are '-', Neg 2 & fall through to normal UDIV
	rcall NEGMANT2

;** Straight 32bit Unsigned ACC1/ACC2 **

UDIV:   rcall	tstzero2        ; test for Div/0
	breq	DIVZERO
	rcall	tstzero1
        breq	UDIVX           ; save lots of time!
	clr	R8
	clr	R9
	clr	R10
	clr	R11
	lsl	R16
	rol	R17
	rol	R18
	rol	R19
	ldi	count, 32

UDIVLP: rol	R8
	rol	R9
	rol	R10
	rol	R11
	cp	R8, R20
	cpc	R9, R21
	cpc	R10, R22
	cpc	R11, R23
	brlo	UDIV2
        sub     R8,R20
	sbc	R9, R21
	sbc	R10, R22
	sbc	R11, R23
	sec
	rjmp UDIV3

UDIV2:  clc
UDIV3:  rol	R16
	rol	R17
	rol	R18
	rol	R19
	dec	count
	brne	UDIVLP
UDIVX:  ret

DIVZERO:                ;error(Div0Msg)
	ret

;** Get the Absolute value in Acc1 **
;** abs(int) **

ABS:    tst     R17
	brpl ABSX
        ser     R18          ; set MSBytes to 0xff
	ser R19
	rjmp NEGMANT1
ABSX:   ret


;** long integer abs((long)) **
		brpl NEGMANT1x

NEGMANT1:	com     R16
		com     R17
		com     R18
		com     R19
		subi	R16,-1
		sbci	R17,-1
		sbci	R18,-1
		sbci	R19,-1
NEGMANT1x:      ret

NEGMANT2:       com	R20
		com	R21
		com	R22
		com	R23
		subi R20, -1
		sbci R21, -1
		sbci R22, -1
		sbci R23, -1
		ret

;******************************************
;** Signed 32bit Mul                     **
SMUL:	tst	R19
	brmi	SMUL1n
	tst	R23
        brpl	UMUL            ; both are Pos, normal
        rcall	NEGMANT2        ; Acc1+, Acc2-, Negate Acc2

SMUL1a: rcall	UMUL            ; Normal Mul
        rjmp	NEGMANT1        ; Negate Results

SMUL1n: rcall	NEGMANT1
	tst	R23
	brpl	SMUL1a

				;both are '-', Neg 2 & fall through to normal UMUL
	rcall	NEGMANT2

;*************************
;** Unsigned 32bit Mul  **
;**                     **
;** Acc1 *= Acc2        **
UMUL:   rcall	tstzero1        ; speeds things Up! if Acc1 == 0
        breq	UMULX           ; if Acc1 == 0, return
        rcall	tstzero2        ; if Acc2 == 0
	brne	UMULOK
        clr	R16           ; yes, x * 0 = 0. If this part is to big, remove.
	clr	R17
	clr	R18
	clr	R19
	ret

UMULOK: clr	R8
	clr	R9
	clr	R10
	clr	R11
        clr	temp            ; used for 'adci mant1x,0'
        ldi	count,32        ; number of bits to proc.

UMULLP: lsl	R8
	rol	R9
	rol	R10
	rol	R11
	rol	R16
	rol	R17
	rol	R18
	rol	R19
	brcc	UMUL1
	add	R8,R20
	adc	R9,R21
	adc	R10,R22
	adc	R11,R23
        adc	R16,temp      ; addi 0+Carry
        adc	R17,temp     ; trickle the Carry on down.
	adc	R18,temp
	adc	R19,temp

UMUL1:  dec	count
	brne	UMULLP

	mov	R16,R8
	mov	R17,R9
	mov	R18,R10
	mov	R19,R11
UMULX:  ret

;********************************
;** atoi() and atol() routines **

_imul10:        mov R21, R17
		mov R22, R18
		mov R23, R19
                rcall _rolmant          ;*2
                rcall _rolmant          ;*4
                add R16,R20         ;*5
		adc R17, R21
		adc R18, R22
                adc R19,R23     ; fall through to final * 10

_rolmant:       rol R17
		rol R18
		rol R19
		ret

_getch:		ld	temp,X+		;temp <- 'ASCII char' use your own
		ret


_junk:  rcall   _getch
        cpi     temp,' '
        breq    _junk
        cpi     temp,0x09               ; Tab?
        breq    _junk
	ret

ATOI:
        ; use atol for both 16bit & 32bit, saves AVR code space

ATOL:	clr R16
	clr R17
	clr R18
	clr R19
	clt                     ; assume Pos
	rcall   _junk
	cpi     temp,'-'        ; is it Neg?
	brne    ATOL2           ; change to ATOLp if you want '+' detection.
	set                     ; yes, flag the op.
	rjmp    ATOL1           ; added bug fix!, used to fall through

ATOLp:  cpi     temp,'+'        ; not really nessec, but some like using '+'
	brne ATOL2
ATOL1:  rcall   _getch
ATOL2:  cpi     temp,'9'+1
	brsh ATOLX
        cpi     temp,'0'
	brlo ATOLX
        rcall   _imul10
        subi    temp,'0'
        add     R16,temp      ; add new digit
        clr     temp            ; just add 0+Carry
	adc R17, temp
	adc R18, temp
	adc R19, temp
	rjmp ATOL1

ATOLX:  brtc    ATOLXX          ; did we encounter a '-'?
        rjmp    NEGMANT1        ; yes, negate the num
ATOLXX: ret




;*******************************************************************
;** Fairly fast, & code eff. ltoa((long)num) and itoa((int)num) **
;** Blanks Leading Zeros **
;** Acc1 is destroyed, you must preserv it yourself **
;* enter 'T' == 0, then UNSIGNED Print else, SIGNED Decimal Print *
;* R14 = 0 nincs bevezeto nulla

ITOA:   clr	R18          ; assume 16 bit Pos num
	clr	R19
        tst	R17          ; test bit 15
	brpl	SLTOA
        ser	R18          ; set the MSB to 0xff
	ser	R19

;* Fall through to 32 bit Signed LTOA *
SLTOA:	set                     ; Signed Print Decimal
	rjmp	LTOA

ULTOA:	clt			; Un-Signed Print Decimal
LTOA:   ldi	XL,low(ASCIIBuf) ;Ascii Buffer Ptr
	ldi	XH,high(ASCIIBuf)

	brtc	LTOA1		; signed/unsigned flag from caller
	tst	R19		; test Sign bit 31
	brpl	LTOA1
	rcall	NEGMANT1	; 2's comp to Pos num
	ldi	temp,'-'
	st	X+,temp		; store the Sign, Buf pointer++

LTOA1:	clr	temp
        or	temp,R16
        or	temp,R17
        or	temp,R18
        or	temp,R19
        ldi	count,9		; 9 rows of 4 columns ea.
	ldi	ZL,low(I1E9)	; Decimal Table below
	ldi	ZH,high(I1E9)
	in	R0,RAMPZ
	push	R0
	lsl	ZL
	rol	ZH
	ldi	R20,0
	rol	R20
	out	RAMPZ,R20

LTOAloop:			; load each sequential table group
	elpm	R20,Z+
	elpm	R21,Z+
	elpm	R22,Z+
	elpm	R23,Z+
        clr	temp		; now test Acc1 w/table in Acc2.


DECDIG1: cp	R16,R20
	cpc	R17,R21
	cpc	R18,R22
	cpc	R19,R23
        brlo	DECDIG2		; can we subtract without borrow?
        sub	R16,R20		; yes
	sbc	R17,R21
	sbc	R18,R22
	sbc	R19,R23
        inc     temp		; inc this each time we can subtract.
        rjmp    DECDIG1		; keep subtracting until underflow!

DECDIG2:
	ori    temp,'0'		; convert to Ascii char
	st	X+,temp         ; store the dig in the SRAM Buffer, and inc 'X' reg

DECDIG4: dec	count           ; have we processed all tables yet?
	brne	LTOAloop

DECDIGX: ori	R16,'0'       ; print the last '0-9' residual digit!
        st	X+,R16        ; save it.

				; * count = 0 here, use it to add ending 'NULL' char. *
        st      X+,count

	pop	R0
	out	RAMPZ,R0

        ret                     ; return to caller!

;** Number to ASCII convertion Table **
I1E9:   .db     0x00,0xca,0x9a,0x3b     ; 1,000,000,000
I1E8:   .db     0x00,0xe1,0xf5,0x05     ;   100,000,000
I1E7:   .db     0x80,0x96,0x98,0        ;    10,000,000
I1E6:   .db     0x40,0x42,0x0f,0        ;     1,000,000
I1E5:   .db     0xa0,0x86,0x01,0        ;       100,000
I1E4:   .db     0x10,0x27,0x00,0        ;        10,000
I1E3:   .db     0xe8,0x03,0x00,0        ;         1,000
I1E2:   .db     0x64,0x00,0x00,0        ;           100
I1E1:   .db     0x0a,0x00,0x00,0        ;            10

.cseg

