;***************************************************************************
;
; File Name		:'mpoint.asm"
; Title			:morph points in 3D
; Date			:2003.09.13.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034,  old: +36-30-9541-658 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:AVR
;
;***************************************************************************
;	D E S C R I P T I O N
;
; Display, shift, rotate and morph points in 3D.
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  	why
;	----	----------  ---		------------------------------------
;	0.01	2003.09.13  VFX		Creation
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	SYSCLK: f=16.000 MHz (T= 62.5 ns)
;*
;***************************************************************************
;
;
;**************************************************************************
;* Hardware Def.
;
;diplay area 320x200
;
.equ	maxx = S1D_DISPLAY_WIDTH
.equ	maxy = 200

.equ	numpts = 120			; number of points

.equ	DMProcess = 0x111

;RAM szukseglet
.equ	tmp_pts1 = numpts*4		; temp. buffer for points x,y : word 


.equ	RAMNeeded = tmp_pts1



;
;***************************************************************************
;**** VARIABLES
.DSEG

DMmempage:	.byte 1
DMmemoffset:	.byte 2


rot_x:		.byte 2		; current rotation around x axis
rot_y:		.byte 2		;                         y axis
rot_z:		.byte 2		;                         z axis
delta_x:	.byte 2		; delta rotation x axis
delta_y:	.byte 2		;                y axis
delta_z:	.byte 2		;                z axis

morph1_ofs:	.byte 2		; offset shape 1 for morphing
morph2_ofs:	.byte 2		;        shape 2
morph_steps:	.byte 2		; number of steps needed for morphing
curr_morph_step:.byte 2		; current morph step [0 ... morph_steps]


sin_x:		.byte 2		; current sin for rot. x axis
cos_x:		.byte 2		;         cos          x
sin_y:		.byte 2		;         sin          y
cos_y:		.byte 2		;         cos          y
sin_z:		.byte 2		;         sin          z
cos_z:		.byte 2		;         sin          z


;***************************************************************************
.ESEG


;***************************************************************************
;**** CODE SEG
;***************************************************************************
.CSEG

.include "points\sin.inc"
.include "points\cube.inc"
.include "points\ball.inc"


DM_Points:
		call	DM_InitScreen

		clr	R18
		ldi	R17,High(tmp_pts1)
		ldi	R16,Low(tmp_pts1)	;required memory

		ldi	R19,Low(DMProcess)
		ldi	R20,High(DMProcess)	;Process number

		call	MmAllocMemory
		brcc	DM_AllocOK
		ret

DM_AllocOK:
		call	MmTranslatePhysicalAddress
		sts	DMmempage,R18
		sts	DMmemoffset+0,R16
		sts	DMmemoffset+1,R17
		call	MmSetPageMap		;memoria belapozva, upper 32k
	
		ret




DM_InitScreen:                     		 ;Clear arena
		call	WaitToDMA
		clr	R16
		clr	R17
		movw	R18,R16
		ldi	R20,low(S1D_DISPLAY_WIDTH)
		ldi	R21,high(S1D_DISPLAY_WIDTH)
		ldi	R22,low(200)
		ldi	R23,high(200)
		clr	R24
		call	DrawBox

		clr	R16			;Draw Controls
		clr	R17
		ldi	R18,low(200)
		ldi	R19,high(200)
		ldi	R20,low(S1D_DISPLAY_WIDTH)
		ldi	R21,high(S1D_DISPLAY_WIDTH)
		ldi	R22,low(40)
		ldi	R23,high(40)
		ldi	R24,0xAA
		call	DrawWin
		nop
		call	StartDMA
		ret




;-----------------------------------------------------------------------
; IN:
;	R17:R16 = ax = px
;	R19:R18 = bx = py
;	R21:R20 = cx = distance point <-> screen
; uses: R0
;
DM_plot_point:
		clr	R0		; R21:R20 = distance point <-> screen
		subi	R20,60
		sbc	R21,R0		; calculate the color of the pixel
		brge	DM_NotNegative
		clr	R20
		clr	R21
DM_NotNegative:
DM_NotZero:
		inc	R0
		lsr	R21
		ror	R20
		brne	DM_NotZero
		tst	R21
		brne	DM_NotZero
		ldi	R20,16
		sub	R20,R0		;R20 contains the color of the pixel

		call	seSetPixel	;(x,y,Color)
		ret



;-----------------------------------------------------------------------
;  get sine and cosine values
; IN:
;	R16 = angle [0..255]
; OUT:
;	R17:R16 = ax = sin(x)
;	R19:R18 = bx = cos(x)
;
;uses:  R0,R20
;
DM_GetSinCos:

	ldi	ZL,low(Sin256*2)
	ldi	ZH,high(Sin256*2)
	ldi	R20,(Sin256/32768)
	out	RAMPZ,R20
	mov	R0,R16			;save angle
	clr	R20
	lsl	R16
	rol	R20			;R20:R16 * 2.. word is 2 bytes
	add	ZL,R16
	adc	ZH,R0
	elpm	R16,Z+
	elpm	R17,Z+			;R17:R16 = sin(x)
	
	ldi	ZL,low(Sin256*2)
	ldi	ZH,high(Sin256*2)
	ldi	R20,(Sin256/32768)
	out	RAMPZ,R20

	ldi	R20,64			; since sine tabel ranges from
	add	R20,R0			; [0 .. 255], so cos(x) = sin(x+64)
	clr	R0
	
	lsl	R20
	rol	R0			;R20:R16 * 2.. word is 2 bytes
	add	ZL,R20
	adc	ZH,R0
	elpm	R18,Z+
	elpm	R19,Z+			;R18:R19 = cos(x)
	
        ret



;-----------------------------------------------------------------------
;   specify x, y, z rotation
; IN:
;	R17:R16 = ax = x rot
;	R19:R18 = bx = y rot
;	R21:R20 = cx = z rot
; OUT:
;
;uses:  R0,R2,R3,R4,R5,R6,R7
;
DM_SetRotation:				; ax = x rot, bx = y rot, cx = z rot
		movw	R2,R16
		movw	R4,R18
		movw	R6,R20
					; range sine table [0 .. 255]

        	call    DM_GetSinCos
        	sts	sin_x+0,R16
		sts	sin_x+1,R17
		sts	cos_x+0,R18
		sts	cos_x+1,R19

		mov	R16,R4
		call    DM_GetSinCos
        	sts	sin_y+0,R16
		sts	sin_y+1,R17
		sts	cos_y+0,R18
		sts	cos_y+1,R19

		mov	R16,R6
	        call    DM_GetSinCos
        	sts	sin_z+0,R16
		sts	sin_z+1,R17
		sts	cos_z+0,R18
		sts	cos_z+1,R19

		movw	R20,R6
		movw	R18,R4
		movw	R16,R2
		ret


;-----------------------------------------------------------------------
;   update rotation params.
; IN:
;	R17:R16 = ax = x rot
;	R19:R18 = bx = y rot
;	R21:R20 = cx = z rot
; OUT:
;
;uses:  R0,R2,R3,R4,R5,R6,R7,R8,R9
;
; add delta_x, delta_y and delta_z
; to rot_x, rot_y and rot_z,
; then calculate the sines and cosines.
;-----------------------------------------------------------------------
;
DM_UpdateRotation:
		movw	R2,R16
		movw	R4,R18
		movw	R6,R20
                                        
		lds	R16,rot_x+0
		lds	R17,rot_x+1
		lds	R8,delta_x+0
		lds	R9,delta_x+1
		add	R16,R8
		adc	R17,R9
		sts	rot_x+0,R16
		sts	rot_x+1,R17

		lds	R18,rot_y+0
		lds	R19,rot_y+1
		lds	R8,delta_y+0
		lds	R9,delta_y+1
		add	R18,R8
		adc	R19,R9
		sts	rot_y+0,R18
		sts	rot_y+1,R19


		lds	R20,rot_z+0
		lds	R21,rot_z+1
		lds	R8,delta_z+0
		lds	R9,delta_z+1
		add	R20,R8
		adc	R21,R9
		sts	rot_z+0,R20
		sts	rot_z+1,R21
		call    DM_SetRotation

		movw	R20,R6
		movw	R18,R4
		movw	R16,R2
		ret

;-----------------------------------------------------------------------
;   Set parameters for morphing.
; IN:
;	R17:R16 = ax = offset shape1
;	R19:R18 = bx = offset shape2
;	R21:R20 = cx = total number of steps
; OUT:
;
;uses:  R0,R2,R3,R4,R5,R6,R7,R8,R9
;
; Both shapes must have the same
; number of points.
; Input: ax = offset shape1.
;        bx = offset shape2.
;        cx = total number of steps

DM_SetMorph:
		sts	morph1_ofs+0,R16
		sts	morph1_ofs+1,R17
		sts	morph2_ofs+0,R18
		sts	morph2_ofs+1,R19
		sts	morph_steps+0,R20
		sts	morph_steps+1,R21
		clr	R0
		sts	curr_morph_step+0,R0
		sts	curr_morph_step+1,R0
		ret


;-----------------------------------------------------------------------
; project points, plot'em and update
;
; IN:
; OUT:
;
;uses:  
;
DM_DrawPoints:					; rotation data.
		call	DM_UpdateRotation	; update the rotation params.

		lds	YL,DMmemoffset+0
		lds	YH,DMmemoffset+1	;Y -> point table [points]

		;        mov     si, [points]		; ds:si -> [points]
		;	 mov     ax, [cs:HiddenPoints]	; es:di -> HiddenPoints tabel

		ldi	XL,low(numpts)
		ldi	XH,high(numpts)

PlotLoop:
		push	XL
		push	XH

		ldd	R16,Y+0
		ldd	R17,Y+1			; get x coord.
		ldd	R18,Y+2
		ldd	R19,Y+3
		ldd	R20,Y+4			; get y coord.
		ldd	R21,Y+5
		ldd	R16,Y+6			; get z coord.

;		rcall    DM_project_point		; project the point onto the screen
		adiw	YL,6
		rcall    DM_plot_point		; plot the point

						;store x coord. for fast clrscr.
						; store y coord.
		pop	XH
		pop	XL
		sbiw	XL,1
		brne	PlotLoop		; any more points?


;		call    FlipPages		; no: flip the pages
;		call    ClrScr			; clear the hidden page.

		ret












;--------------------------------------------------------------------------
;DM_project_point
;
; project a point to screen using x,y,z rotations

; returns: ax = x coord.
;          bx = y coord.
;          cx = distance to screen.
;
; IN:
;	R17:R16 = x = ax
;	R19:R18 = y = bx
;	R21:R20 = z = cx
;
DM_project_point:
                                        
;        ARG     z:word, y:word, x: word           
                                        
		ret