;***************************************************************************
;
; File Name		:'sl811hs.asm"
; Title			:SL811HS USB Master/Slave Controller Driver
; Date			:2003.07.06.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034,  old: +36-30-9541-658 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:ATmega128
;
;***************************************************************************
;	D E S C R I P T I O N
;
; AVRPAC USB Manager
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  	why
;	----	----------  ---		------------------------------------
;	0.01	2003.07.06  VFX		Creation
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	SYSCLK: f=16.000 MHz (T= 62.5 ns)
;*
;***************************************************************************
;* Const Def


;**************************************************************************
;* Hardware Def.

.equ	USBIRQPORT = PORTE
.equ	USBIRQDIR  = DDRE
.equ	USBIRQ	   = 4



.equ	EP0Control	= 0x00
.equ	EP0Address	= 0x01
.equ	EP0XferLen	= 0x02
.equ	EP0Status	= 0x03
.equ	EP0Counter	= 0x04

.equ	USBAHostDevAddr = 0x04
.equ	USBControl1	= 0x05
.equ	USBIntEnable	= 0x06
.equ	USBBHostControl	= 0x08
.equ	USBBHostBase	= 0x09
.equ	USBBHostBaseLen	= 0x0A
.equ	USBBHostPIDStat	= 0x0B
.equ	USBBHostDevAddr = 0x0C
.equ	USBBHostTransCnt= 0x0C
.equ	USBStatus	= 0x0D
.equ	SOFCntLOW	= 0x0E
.equ	HWRevision	= 0x0E
.equ	SOFCntHIGH_Cntr2= 0x0F
.equ	USBMemoryBuffer	= 0x10


;***************************************************************************
;**** VARIABLES
.DSEG


;***************************************************************************
.ESEG


;***************************************************************************
;**** CODE SEG
;***************************************************************************
.CSEG

USB_Init:

		ldi	R16,SOFCntHIGH_Cntr2
		sts	ADR_USBA,R16
		nop
		nop
		nop
		ldi	R16,0xAE
		sts	ADR_USBD,R16		;SL811 in Host mode

						;Enable power to USB (not implemented)

		ldi	R16,Low(2)
		sts	SCNT1+0,R16
		ldi	R16,High(2)
		sts	SCNT1+1,R16		;varakozas max >25ms
USB_I1:
		lds	R16,SCNT1+0
		lds	R17,SCNT1+1
		or	R16,R17
		breq	USB_I1
						;USB Memory test (not implemented)
		
		ldi	R16,USBStatus
		sts	ADR_USBA,R16
		ldi	R16,255
		nop
		nop
		nop
		sts	ADR_USBD,R16		;Clear All Pending Interupt

						;init ATmega External interrupt
		cbi	USBIRQDIR,USBIRQ
		sbi	USBIRQPORT,USBIRQ	;Input pull-up


		in	R16,EICRB
		andi	R16,0b11111100
		ori	R16,0b00000010
		out	EICRB,R16		;INT4 falling edge

		ldi	R16,0x10
		out	EIFR,R16		;Pending INT4 interrupt cleared
		in	R16,EIMSK
		ori	R16,0x10
		out	EIMSK,R16		;INT4 enabled


		ldi	R16,USBIntEnable
		sts	ADR_USBA,R16
		nop
		nop
		nop
		ldi	R16,0b00100011		;Interrupt enabled
		sts	ADR_USBD,R16
		nop
		nop
		nop

		ldi	R16,USBControl1
		sts	ADR_USBA,R16
		nop
		nop
		nop

		ldi	R16,0x48		;USB Bus Reset, Set full speed
		sts	ADR_USBD,R16

		nop
		nop
		nop
		nop

		ldi	R16,USBControl1
		sts	ADR_USBA,R16
		nop
		nop
		nop
		ldi	R16,0x00		;USB normal, USB & SOF Disabled
		sts	ADR_USBD,R16
		nop
		nop
		nop


		ldi	R16,SOFCntHIGH_Cntr2
		sts	ADR_USBA,R16
		nop
		nop
		nop
		ldi	R16,0xAE
		sts	ADR_USBD,R16		;SL811 in Host mode
		nop
		nop
		nop
		ldi	R16,SOFCntLOW
		sts	ADR_USBA,R16
		nop
		nop
		nop
		ldi	R16,0x0E
		sts	ADR_USBD,R16
		nop
		nop
		nop


		ldi	R16,EP0Control
		sts	ADR_USBA,R16
		nop
		nop
		nop
		ldi	R16,0x05
		sts	ADR_USBD,R16		;control
		nop
		nop
		nop

		ldi	R16,0x10
		sts	ADR_USBD,R16		;base
		nop
		nop
		nop
	
		ldi	R16,64
		sts	ADR_USBD,R16		;len
		nop
		nop
		nop

		ldi	R16,0
		sts	ADR_USBD,R16		;PID?? EP??
		nop
		nop
		nop

		ret



;***********************************************
; INT4 - USB IRQ
;
EXT_INT4:

		cbi	LED1_PORT,LED1		;Green LED on

		RETI



