;***************************************************************************
;
; File Name		:'spiic.asm"
; Title			: ATmega8 - ATmega128 SPI Interconect by CPLD
; Date			:2003.07.11.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034,  Old: +36-30-9541-658 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:ATmaga128
;
;***************************************************************************
;	D E S C R I P T I O N
;
; using INT5 for external interrupt pin
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who		why
;	----	----------  ---		--------------------------------------------
;	0.01	2003.07.11  VFX		Creation
;
;***************************************************************************
;
;Hardware
;***************************************************************************
;*
			

;**************************************************************************
;* Const Def


;********************************************************************
;

.equ	ICRx_Buffer_Length = 16


;***************************************************************************
.DSEG

ICRx_Buffer:
ICRx_Head:	.byte 1
ICRx_Tail:	.byte 1
ICRx_Data:	.byte ICRx_Buffer_Length


;*************************************************************************
;*  Text Const
;*
.CSEG


;************************************************************************
;** SPIIC_Init
;**
;**
;************************************************************************
SPIIC_Init:	ldi	R16,0
		sts	ICRx_Tail,R16
		sts	ICRx_Head,R16
		sbi	PORTE,5
		cbi	DDRE,5			;INT5 input pull-up
		
		in	R16,EICRB
		andi	R16,0b11110011
		out	EICRB,R16

		ldi	R16,32
		out	EIFR,R16		;Pendng interrupt clear
		in	R16,EIMSK
		ori	R16,32
		out	EIMSK,R16		;Int5 Enabled

		ldi	R16,0
		sts	ADR_SPID,R16		;RTS=0 johet az adat

		ret



;*******************************************
;* SPI IC RX COMPLET
;*******************************************
EXT_INT5:	push	ZL
		push	ZH
		in	ZL,SREG			; preserve main OS status reg.
		push	ZL
		push	R16
		push	R2


;		call	TogleLED

		clr	R2
		ldi	ZL,low(ICRx_Data)
		ldi	ZH,high(ICRx_Data)
		lds	R16,ICRx_Head
		add	ZL,R16
		adc	ZH,R2


		lds	R2,ADR_SPID		; get the incomming char
		st	Z,R2			; Buf[head] = UDR

		ldi	ZL,32
		out	EIFR,ZL			;Pendng interrupt clear

		inc	R16
		cpi	R16,ICRx_Buffer_Length
		brlo	ICinx
		clr	R16			; Circle around to the first buffer pos.
ICinx:		sts	ICRx_Head,R16


		ldi	ZL,ICRx_Buffer_Length
		lds	R2,ICRx_Tail
		sub	R2,R16
		breq	ICPSC4
		brcs	ICPSC2
		clr	ZL
ICPSC2:
		add	ZL,R2
		dec	ZL
ICPSC4:		cpi	ZL,1			;legalabb 1 hely maradjon ures!
		brsh	ICIntExit

		ldi	R16,1
		sts	ADR_SPID,R16		;RTS=1 nincs tobb hely


ICIntExit:	pop	R2
		pop	R16
		pop	ZL
		out	SREG,ZL			; restore previous status reg
		pop	ZH
		pop	ZL
		reti


;************************************************
;* The SPI IC RxD FIFO manager			*
;* In: -                                        *
;* Out C=1, akkor R0= rx-ed char		*
;************************************************
ICGetByte:	clr	R0
		lds	R18,ICRx_Tail
		lds	R19,ICRx_Head
		cp	R19,R18
		breq	ICGetBytexit
		ldi	ZL,low(ICRx_Data)		; Pointer to the rxbuffer (FIFO)
		ldi	ZH,high(ICRx_Data)
		add	ZL,R18
		adc	zh,R0			; add address offset...  z = &Buf[Tail]
		ld	R0,Z			; R0 = Buf[Tail++]
		inc	R18
		cpi	R18,ICRx_Buffer_Length	; test for the end of the data buffer
		brlo	ICGbx
		clr	R18			; Circle around to the first buffer pos.
ICGbx:		sts	ICRx_Tail,R18

		ldi	R16,0
		sts	ADR_SPID,R16		;RTS=0 johet az adat

		sec				;Valid Char
		ret
ICGetBytexit:
		clc				; Return 'No' bytes in yet.
		ret

