;***************************************************************************
;
; File Name		:'terminal.asm"
; Title			:RS-232 Terminal
; Date			:2001.12.26.	Lastmod.:[2003.02.08.]
; Version		:1.0.2
; Support telephone	:+36-70-333-4034
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		: ***
;
;***************************************************************************
;	D E S C R I P T I O N
;
;  RS-232 alapu terminal
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  	why
;	----	----------  ---		--------------------------------------------
;	0.01	2001.12.26  VFX		Creation
;	1.00	2003.02.02  VFX		Redesign all functions
;
;***************************************************************************
;Hardware
;***************************************************************************
;*


;**************************************************************************
;* Const Def

.equ	CMD_Buffer_Length = 32

.equ	Term_EchoHEX_Mode	= 0b00000001	;Terminal loopback mode HEX
.equ	Term_EchoASCII_Mode	= 0b00000010	;Terminal loopback mode ASCII
.equ	Term_NoOkMsg		= 0b00000100	;No "Ok." measage


.equ	Term_Rec_Xmodem	= 0b01000000		;Terminal XMODEM Receive mode


;***************************************************************************
.DSEG

CMD_Len:	.byte 1				;a CMD aktualis hossza
CMD_Pos:	.byte 1				;Curzor pozicio a CMD stringen belul
CMD_Data:	.byte CMD_Buffer_Length		;CMD buffer 'string'
Term_Stat:	.byte 1		 		;Terminal status


;*************************************************************************
;*  Text Const
;*
.CSEG


Init_Term:	clr	R16			;normal Terminal munkodes
		sts	CMD_Len,R16
		sts	CMD_Pos,R16
		sts	Term_Stat,R16
		ret


;***********************************************
; Terminal, Main

Terminal:
		lds	R16,Term_Stat
		andi	R16,Term_Rec_Xmodem	;XMODEM Mode Receive
		breq	Term01
		jmp	Xmodem_Rec		;Itt Xmodem modban vagyunk

Term01:
		rcall	Rs232GetByte
		brcs	Term1
		ret

Term1:
		lds	R17,Term_Stat
		andi	R17,Term_EchoHEX_Mode	;TEST mode
		brne	Term1_EchoHex

		lds	R17,Term_Stat
		andi	R17,Term_EchoASCII_Mode
		breq	Term1_2

Term1_EchoASCII:
		mov	R16,R0 			;itt echo modban vagyunk R0-t egybol kuldjuk vissza
		cpi	R16,0x1B
		breq	term1_01		;ESC | CAN  kilepes
		cpi	R16,CAN
		brne	term1_02
term1_01:
		lds	R17,Term_Stat
		andi	R17,255-Term_EchoASCII_Mode
		sts	Term_Stat,R17
		rjmp	Terminal

term1_02:
		rcall	SendChrW
		rjmp	Terminal



Term1_EchoHex:
		mov	R16,R0 			;itt echo modban vagyunk R0-t egybol kuldjuk vissza
		cpi	R16,0x1B
		breq	term1_03		;ESC kilepes
		cpi	R16,CAN
		brne	term1_04
term1_03:
		lds	R17,Term_Stat
		andi	R17,255-Term_EchoHEX_Mode
		sts	Term_Stat,R17
		rjmp	Terminal
term1_04:
		call	HEXtoUART
		ldi	R16,32
		call	SendChrW
		rjmp	Terminal



Term1_2:	mov	R16,R0		;Normal terminal mode
		cpi	R16,$7b		;<'z'
		brcc	Term2
		cpi	R16,$61		;<'a'
		brcs	Term2
		CBR	R16,32		;kisbetu -> NAGYBETU konverzio!!

Term2:		mov	R0,R16		;csak nagybetu jo!!!!!
		cpi	R16,CR		;Enter?
		breq	Term3		;IGEN -> Parancsfeldolgozas
		cpi	R16,LF		;Line feed?
		breq	Term31
		cpi	R16,BS		;BackSpace?
		breq	CMDDelChar


		rcall	PutCharToCMD
                brcc	Terminal
		rcall	CMDTooLong	;Command to long!
		sec
		rjmp	CMDExit

Term3:		rcall	PutCharToCMD
                brcc	CMDProc1
		rcall	CMDTooLong	;Command to long!
		sec
		rjmp	CMDExit

Term31:					;LF volt
		lds	R16,CMD_Pos		;Command Line Curs Pos
		cpi	R16,0
		ldi	R16,CR
		mov	R0,R16
		brne	Term3		;ha nem ures, akkor kicsereljuk CR-re
		rjmp	Terminal

CMDProc1:	rjmp	CMDProcess		;Command line Process

CMDDelChar:                     		;Charakter torles
						;ne ez meg nincs megirva
		lds	R16,CMD_Pos		;Command Line Curs Pos
		cpi	R16,0
		breq	CMDDelC01
		dec	R16
		sts	CMD_Pos,R16
		lds	R17,CMD_Len		;Command Line Lenght
		dec	R17
		sts	CMD_Len,R17
CMDDelC01:
		rjmp	Terminal


;******************************
;* Put char to CMD buffer
;* In: R0 - char
;* Out c=1 no more space

PutCharToCMD:	ldi	ZL,Low(CMD_Data)
		ldi	ZH,High(CMD_Data)
		clr	R1
		LDS	R16,CMD_Pos		;Command Line Curs Pos
		lds	R17,CMD_Len		;Command Line Lenght
		cpi	R17,CMD_Buffer_Length
		brsh	P_err			;Pos>=CMD_Buffer_Length, nincs tobb hely
		cp	R16,R17
		brne	OverWrite		;Ha pos<>Len, akkor feluliras!

		add	ZL,R17
		adc	ZH,R1
		inc	R17
		rjmp	WriteCMD

OverWrite:	add	ZL,R16
		adc	ZH,R1

WriteCMD:	st	Z,R0
		sts	CMD_Len,R17
		inc	R16
		sts	CMD_Pos,R16		;uj pozicio eltarolasa
		clc				;nincs hiba sima tarolas volt
		ret

P_err:		sec				;hiba tarolas nincs
		ret




ESCSpace:	ld	R16,X+
		cpi	R16,' '			;Space kimarad
		breq	ESCSpace
		cpi	R16,HT
		breq	ESCSpace		;horiz.tab is kimarad
		ret



CMDProcess:	ldi	XL,Low(CMD_Data)
		ldi	XH,High(CMD_Data)

		ldi	ZL,Low(Tm_Commands*2)
		ldi	ZH,High(Tm_Commands*2)	;Parancstablazat cime
		ldi	R16,(Tm_Commands/32768)	;Low/High part of FLASH
		out	RAMPZ,R16

LeaderZero:	rcall	ESCSpace
		cpi	R16,LF
		breq	LeaderZero		;Line feed is kimarad

		sbiw	XL,1
		mov	R4,XL
		mov	R5,XH			;elmentjuk az elso ertekes karakter cimet


nextCMD:	elpm	R0,Z+
		tst	R0			;lehivjuk a tabla kovetkezo lelemet
		brne	VanMegCMD		;ha 0, akkor nincs ilyen parancs
		rjmp	CMDUnk			;ismeretlen parancs hibauzenet

VanMegCMD:	ld	R16,X+			;parancs kovetkezo karaktere
		cp	R16,R0
		brne	Nemstimmel
		elpm	R0,Z+			;kovetkezo karakter a sztringben
		tst	R0
		brne	VanMegCMD

						;PARANCS megvan!!!
		adiw	ZL,1			;tablaban a szoveg vegere ertunk
		andi	ZL,254			;tehat jo a parancs
		elpm	R0,Z+			;Z-t eltoljuk a vegrehajto rutin cimehez
		push	R0
		elpm
		push	R0			;STACKban vegrehajto rutin cime
		mov	ZL,XL
		mov	ZH,XH			;Z reg a parancs tobbi resze
		ret				;Ugras a vegrehajto reszre


Nemstimmel:	mov	XL,R4			;itt kell eldonteni miert nem stimmel
		mov	XH,R5			;pointer visszaallitasa

SrchStr:	elpm	R0,Z+			;megkeressuk a sztring veget
		tst	R0
		brne	SrchStr
		adiw	ZL,1
		andi	ZL,254			;csak a paros cim jo
		adiw	ZL,2			;kovetkezo sztring elejere allunk
		rjmp	nextCMD


CMDUnk:		rcall	CMDUnknown
		sec
		rjmp	CMDExit

SyntErr:	rcall	SyntaxErr
		sec
		rjmp	CMDExit

CMDOK:
		lds	R16,Term_Stat
		andi	R16,Term_NoOkMsg
		brne	CMDDone

		rcall	CMDOkMsg
CMDDone:	clc
CMDExit:	clr	R16
		sts	CMD_Len,R16
		sts	CMD_Pos,R16		;alaphelyzet
		ret

;******************************************
;** E R R O R S
;*******************************************

;******************************************
; Command Too Long massege

CMDTooLong:	ldi	R16,Low(LongCMD)
		ldi	R17,High(LongCMD)
		rjmp	SendError

;******************************************
; Command Too Long massege

CMDUnknown:   	ldi	R16,Low(UnknownCMD)
		ldi	R17,High(UnknownCMD)
		rjmp	SendError

;******************************************
; Syntax Error massege

SyntaxErr:   	ldi	R16,Low(StxErr)
		ldi	R17,High(StxErr)
		rjmp	SendError


CMDOkMsg:   	ldi	R16,Low(CMDAck)
		ldi	R17,High(CMDAck)
		rjmp	SendMSG


SendError:	push	R16
		push	R17
		ldi	R16,Low(Errx)
		ldi	R17,High(Errx)
		rcall	SendStrW
		pop	R17
		pop	R16
SendMSG:	rcall	SendStrW
		ret

;********************************************************
; TERMINAL PARANCSOK VEGREHAJTASA
;********************************************************



;****************************************
;** TERMINAL HELP
;**
;
Tm_Help:
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr
		ldi	R16,Low(HelpStr)
		ldi	R17,High(HelpStr)
		rcall	SendStrW
		rjmp	CMDDone

;*************************************************
;** FLASH physical Page Read XXXX in hexa  xxxx= physical address (in word) / 128
;**
;
TM_ReadPage:
		rcall	Get2Digit
		brcs	SyntErr1
		mov	R2,R0			;R2 = hi part of sector
		rcall	Get2Digit		;R0 = low part of sector
		brcs	SyntErr1
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr1
		mov	R11,R0
		mov	R12,R2
		ldi	R23,1			;csak egy blokkot nyom ki
		clr	R13
		rcall	PutBlockxxf
		rjmp	CMDDone


PutBlockxxf:	out	RAMPZ,R12
		clr	R10			;R11:R10 Pointer byte-ban

PutBlock1f:	push	R23

		clr	R2
		clr	R1

		ldi	ZL,16
putblokkf:	push	ZL

		ldi	R25,16
		ldi	XL,low(CMD_Data)
		ldi	XH,high(CMD_Data)

		mov	ZL,R10
		mov	ZH,R11

pblokk1f:    	elpm	R0,Z+
	       	st	X+,R0
	 	dec	R25
		brne	pblokk1f

		mov	R10,ZL
		mov	R11,ZH

		ldi	XL,low(CMD_Data)
		ldi	XH,high(CMD_Data)

		rcall	DumpAddr

		ldi	R16,0x10
		clr	R25
		add	R1,R16
		adc	R2,R25        		;Addr offest+ = 16 byte


		pop	ZL
		dec	ZL
		brne	putblokkf

		ldi	R16,CR
		rcall	SendChrW

		pop	R23
		dec	R23
		brne	PutBlock1f

		ret

SyntErr1:	rjmp	SyntErr

;*************************************
;* Update User FLASH
Tm_Upgrade:
		rcall	ESCSpace			;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr1
	   ;	jmp	0xF046				;Elvileg attol updatelnie kell
		jmp	SyntErr1


;*************************************
TM_ReadRTC:
		rcall	ESCSpace			;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr0
		call	RTC_ReadALL
		call	WaitToTWI

		ldi	XL,Low(IO_CTR_RTC)
		ldi	XH,High(IO_CTR_RTC)
		call	DumpRow
		call	DumpRow
		rjmp	CMDDone



;*************************************
TM_WriteRTC:
		rcall	ESCSpace			;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr0
		call	RTC_Start
		rjmp	CMDOk


SyntErr0:	rjmp	SyntErr


;*****************************************
;
TM_GetTime:
			rcall	ESCSpace			;SPACE kihagy
	       		cpi	R16,CR
	      		brne	SyntErr0
			call	RTC_GetTime
			call	PrintASCII
			ldi	R16,CR
			rcall	SendChrW
			rjmp	CMDDone


;***********************************
; Set Time parameter ST hh:mm:ss
;
Tm_SetTm:    		rcall	ESCSpace	   		;SPACE kihagy
	       		cpi	R16,CR
	      		breq	SyntErr1
			sbiw	XL,1

		ldi	R17,3
		rcall	Tm_SetDTchk
		brcs	Tm_SetDTerr
		swap	R16
		mov	R4,R16				;hours tens
		ldi	R17,10
		rcall	Tm_SetDTchk
		brcs	Tm_SetDTerr
		or	R4,R16				;R4 = hours

		ld	R16,X+
		cpi	R16,':'
		brne	Tm_SetDTerr			;Check for ':'

		ldi	R17,6
		rcall	Tm_SetDTchk
		brcs	Tm_SetDTerr
		swap	R16
		mov	R3,R16				;min tens
		ldi	R17,10
		rcall	Tm_SetDTchk
		brcs	Tm_SetDTerr
		or	R3,R16				;R3 = min

		ld	R16,X+
		cpi	R16,':'
		brne	Tm_SetDTerr			;check for ':'

      		ldi	R17,6
		rcall	Tm_SetDTchk
		brcs	Tm_SetDTerr
		swap	R16
		mov	R2,R16
		ldi	R17,10
		rcall	Tm_SetDTchk
		brcs	Tm_SetDTerr
		or	R2,R16

		rcall	ESCSpace	   		;SPACE kihagy
		cpi	R16,CR
		brne	Tm_SetDTerr

		call	RTC_SetTime
		rjmp	CMDOk

Tm_SetDTerr:	rjmp	SyntErr


Tm_SetDTchk:	ld	R16,X+
		cpi	R16,CR
		breq	Tm_Setchkerr
		subi	R16,'0'
		brcs	Tm_Setchkerr
		cp	R16,R17
		brsh	Tm_Setchkerr
		clc
		ret
Tm_Setchkerr:	sec
		ret


;***********************************
; Set Date parameter SD yyyy-mm-dd
; Out: R5 - Century bit R4:R3:R2 datum BCD

Tm_SetDt:  		rcall	ESCSpace		;SPACE kihagy
	       		cpi	R16,CR
	      		breq	Tm_SetDTerr
			sbiw	XL,1

		ldi	R17,3
		rcall	Tm_SetDTchk
		brcs	Tm_SetDTerr			;year 1900 or 2000
		swap	R16
		mov	R5,R16
		ldi	R17,10
		rcall	Tm_SetDTchk
		brcs	Tm_SetDTerr			;year 1900 or 2000
		or	R16,R5
		clr	R5
		cpi	R16,0x19
		breq	TM_SD1
		inc	R5
		cpi	R16,0x20
		brne	Tm_SetDTerr			;year <> 1900  or 2000 invalid
TM_SD1:							;R5 = 0 or 1 Century bit , XX. or XXI. Century

		ldi	R17,10
		rcall	Tm_SetDTchk
		brcs	Tm_SetDTerr
		swap	R16
		mov	R4,R16
		ldi	R17,10
		rcall	Tm_SetDTchk
		brcs	Tm_SetDTerr
		or	R4,R16				;R4 = years 00-99

		ld	R16,X+
		cpi	R16,'-'
		brne	Tm_SetDTerr
 		ldi	R17,2
		rcall	Tm_SetDTchk
		brcs	Tm_SetDTerr
		swap	R16
		mov	R3,R16
		ldi	R17,10
		rcall	Tm_SetDTchk
		brcs	Tm_SetDTerr
		or	R16,R3				;R3= month 1..12
		tst	R16
		breq	Tm_SetDTerr			;month 00 invalid
		cpi	R16,0x13
		brsh	Tm_SetDTerr			;month > 12 invalid
		mov	R3,R16				;R3= month

		ld	R16,X+
		cpi	R16,'-'
		brne	SyntErr3

      		ldi	R17,4
		rcall	Tm_SetDTchk
		brcs	SyntErr3
		swap	R16
		mov	R2,R16

		ldi	R17,10
		rcall	Tm_SetDTchk
		brcs	SyntErr3
		or	R16,R2
		cpi	R16,0
		breq	SyntErr3			;day 00 invalid
		cpi	R16,0x32
		brsh	SyntErr3			;day>31 invalid
		mov	R2,R16				;R2= day

  		rcall	ESCSpace		;SPACE kihagy
       		cpi	R16,CR
      		brne	SyntErr3

		call	RTC_SetDate
		rjmp	CMDOk

SyntErr3:	rjmp	SyntErr



;******************************************
;* Get DATE
;
TM_GetDate:
			rcall	ESCSpace			;SPACE kihagy
	       		cpi	R16,CR
	      		brne	SyntErr3
			call	RTC_GetDate
			call	PrintASCII
			ldi	R16,CR
			call	SendChrW
			rjmp	CMDDone





;*************************************************
;** SMART MEdia Type
;**
;
TM_SM_ReadID:
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr3

		lds	R0,ADR_SMMODE
		call	BinToUART
		ldi	R16,CR
		call	SendChrW
		call	PrintSMType
		rjmp	CMDOk



;*************************************************
;** SMART physical Page Read XXXX in hexa
;**
;
TM_SM_ReadPage0:
		rcall	Get2Digit
		brcs	SyntErr3
		mov	R2,R0			;R2 = hi part of sector
		rcall	Get2Digit		;R0 = low part of sector
		brcs	SyntErr3
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr3
		mov	R11,R0
		mov	R12,R2
		ldi	R23,1			;csak egy blokkot nyom ki
		clr	R13
		rcall	PutBlockxx
		rjmp	CMDDone


PutBlock:	ldi	R23,32
PutBlockxx:	clr	R10
		call	SM_RD_Page
		push	R16
		ldi	R16,SMIL_RM_ReadData | SMIL_ResetECCLogic		;Reset ECC Logic
		sts	ADR_SMMODE,R16
		ldi	R16,SMIL_RM_ReadData | SMIL_RWwithECC			;Read With ECC
		sts	ADR_SMMODE,R16
		pop	R16

PutBlock1:	push	R23

		clr	R2
		clr	R1

		ldi	ZL,33
putblokk:	push	ZL

		cpi	ZL,1
		brne	putb1212
		push	R16
		ldi	R16,SMIL_RM_ReadData			;Read Without ECC when last blokk
		sts	ADR_SMMODE,R16
		pop	R16


putb1212:
		ldi	R25,16
		ldi	XL,low(CMD_Data)
		ldi	XH,high(CMD_Data)

pblokk1:    	lds	R16,ADR_SMDATA
	       	st	X+,R16
	 	dec	R25
		brne	pblokk1

		ldi	XL,low(CMD_Data)
		ldi	XH,high(CMD_Data)


		rcall	DumpAddr

		ldi	R16,0x10
		clr	R25
		add	R1,R16
		adc	R2,R25        		;Addr offest+ = 16 byte


		pop	ZL
		dec	ZL
		brne	putblokk

		ldi	R16,CR
		call	SendChrW

		pop	R23
		dec	R23
		brne	PutBlock1


		lds	R0,ADR_ECC0			;ECC kirattatasa
		call    HexToUART
		lds	R0,ADR_ECC1
		call    HexToUART
		lds	R0,ADR_ECC2
		call    HexToUART

		ldi	R16,SMIL_Standby
		sts	ADR_SMMODE,R16			; CE = "H"
		ret

SyntErr4:	rjmp	SyntErr

;**********************************************************
;** Logikai clustert ir ki 0-val kezdodoen
;**
;
TM_LogicalCluster:
		rcall	Get2Digit
		brcs	SyntErr4
		mov	R2,R0			;R2 = hi part of sector
		rcall	Get2Digit		;R0 = low part of sector
		brcs	SyntErr4
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr4

		mov	R10,R0
		mov	R11,R2
		call	SM_SearchCluster
		brcs	TMLErr

		mov	R20,R11
		mov	R21,R12
		rcall	PrintR21R20
		rcall	PutBlock
TMLErr:
		rjmp	CMDOK


;**********************************************************
;** Logikai szektort ir ki 0-val kezdodoen
;**
;
TM_LogicalSector:
		rcall	Get2Digit
		brcs	SyntErr4
		mov	R2,R0			;R2 = hi part of sector
		rcall	Get2Digit		;R0 = low part of sector
		brcs	SyntErr4
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr4

		mov	R10,R0
		mov	R11,R2
		clr	R12

		call	SM_SearchSector
		ldi	R23,1
	  	rcall	PutBlockxx

		rjmp	CMDOK

;************************************************
;
TM_GSM:		sbiw	XL,2			;kell az "AT" resz is
TM_GSM1:	ld	R16,X+
		push	XL
		push	XH
		push	R16
		call	Send0ChrW
		pop	R16
		pop	XH
		pop	XL
		cpi	R16,CR
		brne	TM_GSM1
		rjmp	CMDDone

;************************************************
;

TM_GSMSTAT:
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr4
		lds	R0,ADR_GSM
		call	BinToUART
		ldi	R16,CR
		call	SendChrW
		rjmp	CMDDone


;************************************************
;

TM_SEDR:
		rcall	Get2Digit
		brcs	SyntErr5
		mov	R2,R0			;R2 = SED Register
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr5

		ldi	R16,VIDEOBASE+7
		sts	ADR_EXHH,R16
		ldi	ZL,low(0xFFE0)		;SED Register Base Addr.
		ldi	ZH,High(0xFFE0)
		clr	R0
		add	ZL,R2
		adc	ZH,R0
		ld	R0,Z
		call	BinToUART
		ldi	R16,CR
		call	SendChrW
		rjmp	CMDDone

SyntErr5:	rjmp	SyntErr



;************************************************
;

TM_SEDW:
		rcall	Get2Digit
		brcs	SyntErr5
		mov	R2,R0			;R2 = SED Register
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,','
		brne	SyntErr5
		rcall	Get2Digit		;R3=parameter
		brcs	SyntErr5
		mov	R3,R0
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr5

		ldi	R16,VIDEOBASE+7
		sts	ADR_EXHH,R16
		ldi	ZL,low(0xFFE0)		;SED Register Base Addr.
		ldi	ZH,High(0xFFE0)
		clr	R0
		add	ZL,R2
		adc	ZH,R0
		st	Z,R3
		ldi	R16,CR
		call	SendChrW
		rjmp	CMDOk


;***********************************
TM_CCFL:
		rcall	GetHexDigit
		brcs	SyntErr5

		ldi	R16,VIDEOBASE+7
		sts	ADR_EXHH,R16
		ldi	ZL,low(0xFFE0+GPIOStatus)	;SED GPIO Control Register
		ldi	ZH,High(0xFFE0+GPIOStatus)
		st	Z,R0

		ldi	R16,CR
		call	SendChrW
		rjmp	CMDOk

;***********************************
TM_CLS:
		rcall	Get2Digit
		brcs	SyntErr5
		mov	R2,R0			;R2 = SED Register
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr6
		mov	R16,R2
		call	ClearVideo
		rjmp	CMDOk

SyntErr6:	rjmp	SyntErr





;************************************************
;
TM_VPOKE:
		rcall	Get2Digit
		brcs	SyntErr5
		mov	R3,R0
		rcall	Get2Digit
		brcs	SyntErr5
		mov	R2,R0			;R3:R2 = Offset
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,','
		brne	SyntErr5
		rcall	Get2Digit		;R4=parameter
		brcs	SyntErr5
		mov	R4,R0
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr5
		call	VPOKE
		rjmp	CMDOk


TM_Scroll:
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr6

		call	ScrollUp
		call	StartDMA
		ldi	R16,CR
		call	SendChrW
		rjmp	CMDOk

TM_GPSDBG:
		rcall	GetHexDigit
		brcs	SyntErr6
		sts	GPSFlag,R0
		ldi	R16,CR
		call	SendChrW
		rjmp	CMDOk



TM_USBR:
		rcall	Get2Digit
		brcs	SyntErr7
		mov	R2,R0			;R2 = USB Register
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr7

		sts	ADR_USBA,R2
		nop
		nop
		nop
		nop
		lds	R0,ADR_USBD
		call	HEXToUART
		ldi	R16,CR
		call	SendChrW
		rjmp	CMDDone

SyntErr7:	rjmp	SyntErr


TM_USBW:
		rcall	Get2Digit
		brcs	SyntErr7
		mov	R2,R0			;R2 = USB Register
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,','
		brne	SyntErr7
		rcall	Get2Digit		;R3=parameter
		brcs	SyntErr7
		mov	R3,R0
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr7

		sts	ADR_USBA,R2
		nop
		nop
		nop
		nop
		sts	ADR_USBD,R3
		ldi	R16,CR
		call	SendChrW
		rjmp	CMDOk



TM_PEEK:
		rcall	Get2Digit
		brcs	SyntErr7
		mov	R3,R0
		rcall	Get2Digit
		brcs	SyntErr7
		mov	R2,R0			;R3:R2 = Offset
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr7
		mov	XL,R2
		mov	XH,R3
		ld	R0,X
		call	BinToUART
		ldi	R16,CR
		call	SendChrW
		rjmp	CMDDone


TM_CALIBRATE:
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr7
		ldi	R16,0x00
		call	ClearVideo		;Video mem over 16383
						;Test ponts:
						;P0 - 32,24   ->  3856
						;P1 - 288,120 -> 19344
						;P2 - 160,216 -> 34640
		ldi	R16,0xF0
		sts	16384+3856,R16
		nop
		nop
		nop
		sts	16384+19344,R16
		nop
		nop
		nop
		sts	16384+34640,R16
		nop
		nop
		nop
		nop

		rjmp	CMDOk





TM_ADC:
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		breq	SyntErr8
		cpi	R16,'A'
		breq	TM_ADCAll
		subi	R16,'0'
		brcs	SyntErr8
		cpi	R17,8
		brsh	SyntErr8
		mov	R0,R16

		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr8

		add	R0,R0
		clr	R1
		ldi	ZL,low(ADCCH0+2)
		ldi	ZH,high(ADCCH0+2)
		add	ZL,R0
		adc	Zh,R1

		ld	R0,-Z
		ld	R1,-Z
		push	R1
		rcall	HexToUART
		pop	R0
		rcall	HexToUART
		ldi	R16,CR
		rcall	SendChrW

		rjmp	CMDDone

TM_ADCAll:
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr8

		ldi	ZL,low(ADCCH0)
		ldi	ZH,High(ADCCH0)
		ldi	R16,8

TM_ADC1:	push	R16
		ld	R1,Z+
		ld	R0,Z+
		push	ZL
		push	ZH
		push	R1
		rcall	HexToUART
		pop	R0
		rcall	HexToUART
		ldi	R16,32
		rcall	SendChrW
		ldi	R16,32
		rcall	SendChrW
		pop	ZH
		pop	ZL
		pop	R16
		dec	R16
		brne	TM_ADC1
		ldi	R16,CR
		rcall	SendChrW
		rjmp	CMDDone

SyntErr8:	rjmp	SyntErr




TM_SetPixel:
		rcall	Get2Digit
		brcs	SyntErr8
		mov	R3,R0
		rcall	Get2Digit
		brcs	SyntErr8
		mov	R2,R0			;R3:R2 = x coordinata
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,','
		brne	SyntErr8
		rcall	Get2Digit
		brcs	SyntErr8
		mov	R5,R0
		rcall	Get2Digit		;R5:R4 - y
		brcs	SyntErr8
		mov	R4,R0
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,','
		brne	SyntErr8
		rcall	Get2Digit		;R6 - c
		brcs	SyntErr8
		mov	R6,R0
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr8

		ldi	R25,VIDEOBASE+0
		sts	ADR_EXLH,R25
		inc	R25
		sts	ADR_EXHL,R25
		inc	R25
		sts	ADR_EXHH,R25

		movw	R16,R2
		movw	R18,R4
		mov	R20,R6
		call	seSetPixel
		rjmp	CMDOk

TM_SetLine:
		rcall	Get2Digit
		brcs	SyntErr8
		mov	R3,R0
		rcall	Get2Digit
		brcs	SyntErr8
		mov	R2,R0			;R3:R2 = x coordinata
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,','
		brne	SyntErr8
		rcall	Get2Digit
		brcs	SyntErr8
		mov	R5,R0
		rcall	Get2Digit		;R5:R4 - y
		brcs	SyntErr8
		mov	R4,R0
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,','
		brne	SyntErr8
		rcall	Get2Digit		;R6 - c
		brcs	SyntErr8
		mov	R6,R0
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr8

		lds	R16,lastx+0
		lds	R17,lastx+1
		lds	R18,lasty+0
		lds	R19,lasty+1
		movw	R20,R2
		movw	R22,R4
		mov	R24,R6
		call	seDrawLine
		call	StartDMA
		rjmp	CMDOk



TM_DRAWBOX:
;****************************************
; R17:R16 - Left (X)
; R19:R18 - Top (Y)
; R21:R20 - Width
; R23:R22 - Height
;     R24 - Color
;
		rcall	Get4Digit
		brcs	SyntErr9
		movw	R10,R2			;R11:R10 - left

		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,','
		brne	SyntErr9
		rcall	Get4Digit
		brcs	SyntErr9
		movw	R8,R2			;R9:R8 - top
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,','
		brne	SyntErr9
		rcall	Get4Digit
		brcs	SyntErr9
		movw	R6,R2			;R7:R6 - width
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,','
		brne	SyntErr9
		rcall	Get4Digit
		brcs	SyntErr9
		movw	R4,R2			;R5:R4 - height
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,','
		brne	SyntErr9

		rcall	Get2Digit
		brcs	SyntErr9
		mov	R3,R0			;R3- color
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr9

		movw	R16,R10
		movw	R18,R8
		movw	R20,R6
		movw	R22,R4
		mov	R24,R3

		call	DrawWin		;DrawBox
		call	StartDMA
		rjmp	CMDOk


;**************************************************
TM_Poke:
		rcall	Get4Digit
		brcs	SyntErr9
		movw	R2,R2			;R3:R2 - address
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,','
		brne	SyntErr9
		rcall	Get2Digit
		brcs	SyntErr9
		mov	R4,R0			;R4- data
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr9

		movw	ZL,R2
		st	Z,R4
		rjmp	CMDOk



SyntErr9:	rjmp	SyntErr


;**************************************************
TM_MEM:
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr9

		call	MmDumpMemoryAreas
		rjmp	CMDDone



;******************************************************
; Allocate memory
;
TM_MmAlloc:
		rcall	Get4Digit
		brcs	SyntErr9
		movw	R2,R2			;R3:R2 - address
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr9
		movw	R16,R2
		clr	R18
		ldi	R19,0xFE
		ldi	R20,0x01
		call	MmAllocMemory
		rjmp	CMDOk



;******************************************************
; Release Allocated memory
; In: R18:R17:R16 - Allocated memory (MCB)
;     R20:R19 - Process number
;
;Out: c= 0 no error
;
;     c= 1 error
;	 R17:R16 - error code
;
TM_MMFree:
		rcall	Get2Digit
		brcs	SyntErr9
		mov	R4,R0			;R4- first digit
		rcall	Get4Digit
		brcs	SyntErr9
		movw	R2,R2			;R3:R2 - address
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr9
		movw	R16,R2
		mov	R18,R4
		ldi	R19,0xFE
		ldi	R20,0x01
		call	MmReleaseMemory
		rjmp	CMDOk


;************************************************
TM_Echo:
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		breq	EchoHex
		cpi	R16,65			;'A'?
		brne	SyntErr9
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr9
EchAscii:	lds	R16,Term_Stat
		ori	R16,Term_EchoASCII_Mode	;Echo ASCII
		rjmp	EchoC

EchoHex:	lds	R16,Term_Stat
		ori	R16,Term_EchoHEX_Mode	;Echo Hex modban
EchoC:		sts	Term_Stat,R16
		rjmp	CMDOk



;************************************************
TM_TMOK:
		rcall	GetHexDigit
		brcs	SyntErr9
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr9
		lds	R16,Term_Stat
		andi	R16,255-Term_NoOkMsg
		tst	R0
		breq	TM_TMOK0
		ori	R16,Term_NoOkMsg
TM_TMOK0:
		sts	Term_Stat,R16
		rjmp	CMDOk



TM_DMx:
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr10

;		call	DM_Points
		rjmp	CMDOk


SyntErr10:	rjmp	SyntErr


;************************************************
TM_USR:
		rcall	Get4Digit		;R3:R2 - address
		brcs	SyntErr10
		rcall	ESCSpace		;SPACE kihagy
		cpi	R16,CR
		brne	SyntErr10
		movw	ZL,R2
		icall
		rjmp	CMDOk






;***********************************
PrintR21R20:
		push	R20
		push	R21
		mov	R9,R20
		mov	R0,R21
		rcall	HexToUART
		mov	R0,R9
		rcall	HexToUART
		ldi	R16,CR
		rcall	SendChrW
		pop	R21
		pop	R20
		ret


;***************************************
; Get 4 digit
; Out c=1 error
;
Get4Digit:
		rcall	Get2Digit
		brcs	SyntErrhh
		mov	R3,R0
		rcall	Get2Digit
		brcs	SyntErrhh
		mov	R2,R0
		clc
		ret



;*******************************
;* xx 2 hexa digitet keres X reg a pointer
;* ha c=0, R0 - a parameter
;*    c=1, syntax error

Get2Digit:
		rcall	ESCSpace		;SPACE kihagy
		rcall	TstHexDigit
		brcs	SyntErrhh
		swap	R16
		andi	R16,0xF0
		mov	R0,R16

		ld	R16,X+
		rcall	TstHexDigit
		brcs	SyntErrhh
		andi	R16,0x0F
		or	R0,R16
		clc
		ret
SyntErrhh:	sec
		ret

;*************************************
;* Get digit 0..1
;* c=0, R0=0..1
;* c=1, syntax or overflow error
GetHexDigit:
		rcall	ESCSpace		;SPACE kihagy
		subi	R16,'0'
		brcs	SyntErrGH
		cpi	R16,2
		mov	R0,R16
		brcc	SyntErrGH		;Pramerer 0..1 lehet
		clc
		ret

SyntErrGH:	sec
		ret


;*************************************
;* Test Digit 0..9
;* c=0, R16=0..9
;* c=1, syntax or overflow error
TestDigit:
		cpi	R16,'0'
		brcs	DigitRossz
		cpi	R16,'9'+1
		brsh	DigitRossz
		subi	R16,'0'
		clc
		ret
DigitRossz:	sec
		ret





;*************************************
;* Get byte  0..255
; R17 = 0, syntax error R0 = 0 , R16 rossz byte
; R17 = 1 van jo ertek , R0=byte, R16 = rossz bye
; R17 = 2 overflow
;* c=1, syntax or overflow error  R16 = rossz karakter
GetByte:	clr	R0
		clr	R17
		rcall	ESCSpace		;SPACE kihagy
		rcall	TestDigit
		brcc	elsodigitjo
		ret
elsodigitjo:	inc	R17
nextDigit:	mov	R5,R0
		brcs	DigitOverFlw
		add	R0,R0
		brcs	DigitOverFlw
		add	R0,R0
		brcs	DigitOverFlw
		add	R0,R5
		brcs	DigitOverFlw
		add	R0,R0			;R0=R0*10
		brcs	DigitOverFlw
		add	R0,R16			;R0=R0+R16
		brcs	DigitOverFlw
		rcall	ESCSpace
		rcall	TestDigit
		brcc	nextDigit
		ret

DigitOverFlw:	inc	R17
		sec
		ret

;---------------------------------------------------------------------------


Ready:		.DB	"Ready>>",0
CMDAck: 	.DB	"Ok.",CR,0,0
Errx:		.DB	"Error:",0,0
LongCMD:	.DB	" Command too long",CR,0,0
UnknownCMD:	.DB	" Unknown Command",CR,0
StxErr:		.DB	" Syntax Error",CR,0,0
Xmodemt:	.DB	"XMODEM listening... ",0,0


;Terminal parancsok
; "parancs",0,address
; a tabla elso eleme mindig az ures parancs legyen ures sor+ENTER

Tm_Commands:	.db	CR,0,low(CMDOK),high(CMDOK)
		.db	"HELP",0,0,low(Tm_Help),high(Tm_Help)
		.db	"H",0,low(Tm_Help),high(Tm_Help)
		.db	"UPDATE",0,0,low(Tm_Upgrade),high(Tm_Upgrade)
		.db	"RP",0,0,low(TM_ReadPage),high(TM_ReadPage)
		.db	"RTCREAD",0,low(TM_ReadRTC),high(TM_ReadRTC)
		.db	"RTCWRITE",0,0,low(TM_WriteRTC),high(TM_WriteRTC)
		.db	"GT",0,0,low(TM_GetTime),high(TM_GetTime)
		.db	"GD",0,0,low(TM_GetDate),high(TM_GetDate)
		.db	"ST",0,0,low(Tm_SetTm),high(Tm_SetTm)
		.db	"SCROLL",0,0,low(TM_Scroll),high(TM_Scroll)
		.db	"SD",0,0,low(Tm_SetDt),high(Tm_SetDt)
		.db	"SEC",0,low(TM_SM_ReadPage0),high(TM_SM_ReadPage0)
 		.db	"SMID",0,0,low(TM_SM_ReadID),high(TM_SM_ReadID)
		.db	"SEDR",0,0,low(TM_SEDR), high(TM_SEDR)
		.db	"SEDW",0,0,low(TM_SEDW), high(TM_SEDW)
		.db	"LC",0,0,low(TM_LogicalCluster),high(TM_LogicalCluster)
		.db	"LS",0,0,low(TM_LogicalSector),high(TM_LogicalSector)
		.db	"AT",0,0,low(TM_GSM),high(TM_GSM)
		.db	"ADC",0,low(TM_ADC),high(TM_ADC)
		.db	"GSMSTAT",0,low(TM_GSMSTAT),high(TM_GSMSTAT)
		.db	"GPSDBG",0,0,low(TM_GPSDBG),high(TM_GPSDBG)
		.db	"CLS",0,low(TM_CLS),high(TM_CLS)
		.db	"CCFL",0,0,low(TM_CCFL),high(TM_CCFL)
		.db	"CALIBRATE",0,low(TM_CALIBRATE),high(TM_CALIBRATE)
		.db	"VPOKE",0,low(TM_VPOKE),high(TM_VPOKE)
		.db	"USBR",0,0,low(TM_USBR), high(TM_USBR)
		.db	"USBW",0,0,low(TM_USBW), high(TM_USBW)
		.db	"PEEK",0,0,low(TM_PEEK), high(TM_PEEK)
		.db	"POKE",0,0,low(TM_Poke), high(TM_Poke)
		.db	"PLOT",0,0,low(TM_SetPixel), high(TM_SetPixel)
		.db	"LINE",0,0,low(TM_SetLine), high(TM_SetLine)
		.db	"BOX",0,low(TM_DRAWBOX), high(TM_DRAWBOX)
		.db	"MEM",0,low(TM_MEM), high(TM_MEM)
		.db	"MMALLOC",0,low(TM_MmAlloc), high(TM_MmAlloc)
		.db	"MMFREE",0,0,low(TM_MMFree),high(TM_MMFree)
		.db	"ECHO",0,0,low(TM_Echo),high(TM_Echo)
		.db	"TERMOK",0,0,low(TM_TMOK),high(TM_TMOK)
		.db	"DMPLOT",0,0,low(TM_DMx),high(TM_DMx)
		.db	"USR",0,low(TM_USR),high(TM_USR)
		.dw	0



HelpStr:	.db	"VFX  - ATmega128 Service Terminal Help ",CR
		.db	"H vagy HELP - ezt a helpet adja",CR
		.db	"UPDATE - FLASH frissites ",CR
		.db	"RP xxxx - Dump FLASH page xxxx=[addr/128] (word) ",CR
		.db	"RTCREAD - Read RTC all byte to buffer",CR
		.db	"RTCWRITE - Write RTC to zero ",CR
		.db	"GT - Get Time hh:pp:mm ",CR
		.db	"GD - Get Date yyyy-mm-dd ",CR
		.db	"ST oo:pp:mm - beallitja az idot",CR
		.db	"SD yyyy-mm-dd - beallitja a datumot",CR
		.db	"SMID - Read SmartMedia Type" ,CR
		.db	"SEC xxxx - Read SmartMedai Sector xxxx ",CR
		.db	"SEDR xx - Read SED xx Register ",CR
		.db	"SEDW xx,yy - Write SED yy to xx Register ",CR
		.db	"PLOT x,y,c - x,y coordinatara kirak egy c szinu pontot ",CR
		.db	"LINE x1,y1,c - utolso x,y pontbol x1,y1 pontba huz egy vonalat C szinnel ",CR
		.db	"BOX x,y, width, height, color - Dobozt rajzol",CR
		.db	"LS xxxx - xxxx logikai szektor kiirasa ",CR
		.db	"LC xxxx - xxxx logikai cluster kiirasa ",CR
		.db	"AT - GSM AT parancsok",CR
		.db	"ADC x - x=0..7 vagy 'A' x ADC csatorna/ossszes olvasasa",CR
		.db	"GSMSTAT - GSM Status byte",CR
		.db	"CCFL x - CCFL ON/OFF ",CR
		.db	"CLS x - Clear Screen 0..3",CR
		.db	"VPOKE xxx,yy - yy byte-ot beirja a VIDEO RAM xxx cimere",CR
		.db	"SCROLL - Scroll 1 line up",CR
		.db	"GPSDBG - GPS Debug info",CR
		.db	"USBR xx - Read USB xx Register ",CR
		.db	"USBW xx - Write USB xx Register",CR
		.db	"PEEK xxxx - A SRAM xxxx cim tartalmat irja ki",CR
		.db	"POKE xxxx,yy - xxxx cimre beirja yy erteket",CR
		.db	"CALIBRATE - TouchScreen Calibralo kepernyo ",CR
		.db	"MEM - memory usage ",CR
		.db	"MMALLOC xxxx - xxxx byte memoriat lefoglal ",CR
		.db	"MMFREE xxxxxx  - felszabaditja az xxxxxx MCBt",CR
		.db	"ECHO A - minden kapott karaktert HEX,ASCII alakban visszakuld",CR
		.db	"TERMOK x - x=1 Terminal nem kuld 'OK' nyugtat",CR
		.db	"DMPLOT - Plot demo ",CR
		.db	"USR xxxx - call xxxx ",CR
		.db	"H vagy HELP - ezt a helpet adja",CR
		.dw	0


