;***************************************************************************
;
; File Name		:'usart0.asm"
; Title			:ATmega128 UART0,  RS-232 komunikacios rutinok
; Date			:2003.06.08.	Lastmod.:[2003.06.08.]
; Version		:1.0.0
; Support telephone	:+36-70-333-4034,  Old: +36-30-9541-658 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:ATmaga128
;
;***************************************************************************
;	D E S C R I P T I O N
;
;	Universal Synchronous and Asynchronous serial Receiver and Transmitter (USART)
;
;Circular serial I/O buffers, with handshake handling for hardware.
;Interupt driven, thus 'transparent' to User functions
;
;You have a head and tail pointer, a constant that tells the size of the array
;
;One thing to note, is that the buffer store routine is only concerned with the
;tail pointer, and the recall routine is only concerned with the head pointer.
;Only the buffer init routine is concerned with both pointers!
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who		why
;	----	----------  ---		--------------------------------------------
;	0.01	2001.07.21  VFX		Creation (Base on 8515)
;	1.00	2003.02.08  VFX		Redesign for Atmega128
;	1.01	2003.06.08  VFX		Redesign for Atmega128 USART0
;
;***************************************************************************
;
; UART0_Init
;
;Hardware
;***************************************************************************
;*
				;RTS pin , Adas keres DSUB9 7.
.equ	SER0_HSO_PORT=PORTE	;Hardware Handshake output port
.equ	SER0_HSO_DIR=DDRE	;Hardware Handshake output control
.equ	SER0_HSO=2		;Hardware Handshake Output bit

				;CTS pin , Vetelkezseg nyugtazo DSUB9 8.
.equ	SER0_HSI_PORT=PORTB	;Hardware Handshake input port
.equ	SER0_HSI_PIN=PINB	;Hardware Handshake input port
.equ	SER0_HSI_DIR=DDRB	;Hardware Handshake input control
.equ	SER0_HSI=0		;Hardware Handshake Input bit


.MACRO HS0_Enabled
		cbi	SER0_HSO_PORT,SER0_HSO		;Set low, HS to off
.endm

.MACRO HS0_Disabled
		sbi	SER0_HSO_PORT,SER0_HSO		;Set Hi for Dis
.endm


;**************************************************************************
;* Const Def


;From the data sheet:BAUD = XTAL/(16*(UBRR+1))
;Rearranging you get:UBRR = XTAL/(16*BAUD) - 1
;
;This would work except that any fraction will be truncated.
;For least error we want to round off the result.
;So we add 1/2 as follows: UBRR = XTAL/(16*BAUD) - 1 + 0.5
;Combining the -1 and the +0.5 we can rewrite this as follows:
;
;UBRR = INT(XTAL/(16*BAUD) - 0.5)

.equ	Rx0_Buffer_Length = 128
.equ	Tx0_Buffer_Length = 32


;***************************************************************************
.DSEG

Rx0_Buffer:
Rx0_Head:	.byte 1
Rx0_Tail:	.byte 1
Rx0_Data:	.byte Rx0_Buffer_Length

Tx0_Buffer:
Tx0_Head:	.byte 1
Tx0_Tail:	.byte 1
Tx0_Data:	.byte Tx0_Buffer_Length


UART0Flg:	.byte 1			;UART Flags


;*************************************************************************
;*  Text Const
;*
.CSEG


;************************************************************************
;** Init_UART0
;** Set up the inital states for the buffers, and make sure the handshake
;** is turned on (Not the Shaddap state), so we can talk.
;**
;**
;************************************************************************
USART0_Init:	ldi	R16,0
		sts	Tx0_Tail,R16
		sts	Tx0_Head,R16
		sts	Rx0_Tail,R16
		sts	Rx0_Head,R16
		sts	UART0Flg,R16
						;Set up Hardware HS I/O bits
		sbi	SER0_HSO_DIR,SER0_HSO	;Set as output
		HS0_Enabled

		cbi	SER0_HSI_PORT,SER0_HSI	;Set as input, Pull-up
		cbi	SER0_HSI_DIR,SER0_HSI

		ldi	R16,High(SYSCLK/(16*BaudSpeed0)-1)
		sts	UBRR0H,R16
		ldi	R16,Low(SYSCLK/(16*BaudSpeed0)-1)
		out	UBRR0L,R16		;BaudRate Gen.

		ldi	R16,0b00000000		;RXCn, TXCn, UDREn, FEn, DORn, UPEn, U2Xn, MPCMn
		out	UCSR0A,R16

		ldi	R16,0b11011000		;RXCIE,TXCIE,UDRIE,RXEN,TXEN,UCSZn2,RXB8,TXB8
		out	UCSR0B,R16

		ldi	R16,0b00000110		;, UMSELn, UPMn1, UPMn0, USBSn, UCSZn1, UCSZn0, UCPOLn
		sts	UCSR0C,R16
		ret


;***********************************************
;* Megnezi, hogy kell-e ujrainditani a Tx muveletet

Tx0Running:	lds	ZL,Tx0_Head
		lds	ZH,Tx0_Tail
		cp	ZL,ZH			;ha azonosak akkor nincs mit tenni
		breq	Tx0Run1
		lds	ZL,UART0Flg
		andi	ZL,FlSending		;Tx folyamatban?
		brne	Tx0Run1			;ha igen nincs mit tenni
		rjmp	USART0_TXC
Tx0Run1:	ret



;***************************************************************************
;***************************************************************************
;***************************************************************************
;**   UART0 TX Functions
;***************************************************************************
;** Calc Tx Buffer Free Space
; In:	-
; Out:	R18 - Tx_Tail
;	R19 - Tx_Head
;	R20 - Free Space in Tx Buffer
; Alt:  R21
;
; Tail < Head -> Free= LEN+Tail-Head-1
; Tail = Head -> Free= LEN
; Tail > Head -> Free= Tail-Head-1
;
FreeInTx0Buff:	lds	R19,Tx0_Head
		lds	R18,Tx0_Tail
		ldi	R20,Tx0_Buffer_Length
		mov	R21,R18
		sub	R21,R19
		breq	FreeInTx01		;T=H, Free= Tx_Buffer_Length
		brcs	FreeInTx02
                clr	R20
FreeInTx02:	add	R20,R21
		dec	R20
FreeInTx01:	ret


;************************************************
;* The RS232 Tx0 FIFO manager Send CHAR		*
;* in: R16 char 				*
;* out c=1 hiba					*
;************************************************
Send0Chr:	rcall	FreeInTx0Buff		;R18,R19-t is beallitja
		cpi	R20,2
		brcs	Exit0wError		;ha <1 nincs elg hely! Hiba!

		clr	R18			;R18=0
		ldi	ZL,low(Tx0_Data)	;Pointer to the txbuffer (FIFO)
		ldi	ZH,high(Tx0_Data)
		add	ZL,R19
		adc	ZH,R18			;add address offset...  z = @Buf[txhead]

		rcall	Store0Char
		sts	Tx0_Head,R19

		rcall	Tx0Running

		clc				; Nincs hiba
		ret
Exit0wError:	sec				;Return with error
		ret


Send0ChrW:	push	ZL
		push	ZH
		rcall	Send0Chr
		pop	ZH
		pop	ZL
		brcs	Send0ChrW
		ret


;***********************************************
; Subrutin for SendROMTXT & SendChar

Store0Char:	st	Z+,R16			;Store char
		inc	R19
		cpi	R19,Tx0_Buffer_Length
		brcs	StoreC01		;TxHead<Tx_Buffer_Length akkor jo
		clr	R19			;TxHead=0
		ldi	ZL,low(Tx0_Data)	;Pointer to the txbuffer (FIFO)
		ldi	ZH,high(Tx0_Data)
StoreC01:	ret


;***********************************************
; UART Tx Interrupt rutinok


;**********************************************
USART0_DRE:	reti


;**********************************************
;* UART TX0 COMPLET
;**********************************************
USART0_TXC:	push	ZL
		push	ZH
		in	ZL,SREG			;preserve main OS status reg.
		push	ZL
		push	R16
		push	R0
		push	R17
		in	ZL,RAMPZ
		push	ZL

		in	R16,UCSR0A
		ori	R16,(1<<TxC0)		;Clear Tx Complete Flag
		out	UCSR0A,R16

		lds	R17,Tx0_Tail
		lds	ZL,Tx0_Head
		cp	R17,ZL
		brne	Van0meg			;ha nem egyenloek, akkor van meg kuldendo

		clr	R16			;Flagek torlese, alapallapot
		STS	UART0Flg,R16
		rjmp	No0More			;itt a vege nincs tobb adat

Van0meg: 	clr	R0
		ldi	ZL,low(Tx0_Data)	;Pointer to the txbuffer (FIFO)
		ldi	ZH,high(Tx0_Data)
		add	ZL,R17
		adc	ZH,R0 			;z = @Buf[Tx_Tail]
		ld	R16,Z+			;R16=Buf[txtail]
		inc	R17			;Tail+1
		cpi	R17,Tx0_Buffer_Length	;buffer vegere ertunk?
                brcs	tx0n1			;meg nem akkor ugras

		ldi	ZL,low(Tx0_Data)	;Pointer to the txbuffer (FIFO)
		ldi	ZH,high(Tx0_Data)
		clr	R17

tx0n1:
		sts	Tx0_Tail,R17		;Save TxTail

More0TXT:	out	UDR0,R16
		lds	R16,UART0Flg
		ori	R16,FlSending		;ujra atvitel van
		STS	UART0Flg,R16

No0More:
		pop	ZL
		out	RAMPZ,ZL
		pop	R17
		POP	R0
		POP	R16
		POP	ZL
		OUT	SREG,ZL
		POP	ZH
		POP	ZL
		RETI





;***************************************************************************
;***************************************************************************
;***************************************************************************
;**   UART RX Functions
;***************************************************************************


;*******************************************
;* UART0 RX COMPLET
;*******************************************
USART0_RXC:	push	ZL
		push	ZH
		in	ZL,SREG			; preserve main OS status reg.
		push	ZL
		push	R16
		push	R2

		clr	R2
		ldi	ZL,low(Rx0_Data)		; ptr to our rxbuffer... z = Buf[rxtail]
		ldi	ZH,high(Rx0_Data)
		lds	R16,Rx0_Head
		add	ZL,R16
		adc	ZH,R2

						;itt ki kellene olvasni a statust, a rafutas, frame error stb miatt

		in	R2,UDR0			; get the incomming char
		st	Z,R2			; Buf[head] = UDR
		inc	R16
		cpi	R16,Rx0_Buffer_Length
		brlo	Rs232inx0
		clr	R16			; Circle around to the first buffer pos.
Rs232inx0:	sts	Rx0_Head,R16
        					;Kiszamolja van-e meg hely a bufferben

		ldi	ZL,Rx0_Buffer_Length
		lds	R2,Rx0_Tail
		sub	R2,R16
		breq	RStxPSC40			;T=H, akkor Kb_ScData_Lenght hely van
		brcs	RStxPSC20
		clr	ZL
RStxPSC20:
		add	ZL,R2
		dec	ZL
RStxPSC40:	cpi	ZL,1			;legalabb 1 hely maradjon ures!
		brsh	RS232IntExit0

                HS0_Disabled			;ha nincs hely tiltjuk a komunikaciot CTS/RTS vonalon

RS232IntExit0:	pop	R2
		pop	R16
		pop	ZL
		out	SREG,ZL			; restore previous status reg
		pop	ZH
		pop	ZL
		reti				; return to normal pgm status.


;************************************************
;* The RS232 RxD FIFO manager			*
;* In: -                                        *
;* Out C=1, akkor R0=RS232 rx-ed char		*
;************************************************
Rs232GetByte0:	clr	R0
		lds	R18,Rx0_Tail
		lds	R19,Rx0_Head
		cp	R19,R18
		breq	Rs232GetBytexit0
		ldi	ZL,low(Rx0_Data)	; Pointer to the rxbuffer (FIFO)
		ldi	ZH,high(Rx0_Data)
		add	ZL,R18
		adc	zh,R0			; add address offset...  z = &Buf[Tail]
		ld	R0,Z			; R0 = Buf[Tail++]
		inc	R18
		cpi	R18,Rx0_Buffer_Length	; test for the end of the data buffer
		brlo	Rs232Gbx0
		clr	R18			; Circle around to the first buffer pos.
Rs232Gbx0:	sts	Rx0_Tail,R18

		sec				;Valid Char
		ret
Rs232GetBytexit0:
		HS0_Enabled			;CTS/RTS atvitel engedelyezett

		clc				; Return 'No' bytes in yet.
		ret

;****************************************
;* And The RS232 GetChar Loop		*
;* wait until a char comes in 		*
;****************************************
GetChar0:	rcall	Rs232GetByte0
		brcc	GetChar0
		ret

